C*****                   ***********************************************
      SUBROUTINE DCOFUN(XA,AMES,ALAM,CK,F,DF,DDF,D3F)
C*****                   ***********************************************
C     COMPUTATION FOURIER TRANSFORM WITH FINITE CK        
C     USING SERIES EXPANSION
C     XA: UNITS PION WAVE-LENGTH
C----------------------------------------------------------------------
C     DF0 = DPHI_C^0(R) = INT_K^INFTY ( PHI_C^0-INTEGRAND)
C
C     DF1 = (D/DXA) DF0, DF2 = (D/DXA)^2 DF0, DF3 = (D/DXA)^3 DF0
C----------------------------------------------------------------------
C     OPTIONS COMPUTATION: 
C     IASYMP=0: VIA RECURRENCE RELATIONS + PADE      ! VERY SUCCESFUL!
C     IASYMP=1: VIA ESTIMATION INTEGRALS             ! REASONABLE    
C----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER( EPS=1.D-04, MAXIT=20)
      DATA ICALL/0/,PIM/138.041D0/,AMF/197.32D0/,IASYMP/0/
      SAVE PI,SRPI,SR2
C
C     POTENTIAL FUNTION DEFINITION
C
      F0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH-EPH*EXH )/(2*XH)
      H0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH+EPH*EXH )/(2*XH)
C
      IF(ICALL.EQ.0) THEN
      PI = DACOS(-1.D0)
      SRPI=DSQRT(PI)
      SR2 =DSQRT(2.D0)
      ULAM = 10*ALAM
      ENDIF
      RATK = CK/ALAM
      RATM = AMES/ALAM
      RATP = ALAM/PIM
      X= (AMES/PIM)*XA
      XLAM=XA*ALAM/PIM
      XLAM2=XLAM*XLAM
      EX = FDEXP(X)
      EXPM = FDEXP(RATM*RATM) 
      DERFCM = FDERFC(-0.5D0*XLAM+RATM)
      DERFCP = FDERFC(+0.5D0*XLAM+RATM)
      ELAM0  = 0.25D0*SRPI*FDEXP(-0.25D0*XLAM2)

c--------------------------------------------------------------
c     I1(n) and I2(n) recursion-method
c--------------------------------------------------------------
c     write(*,*) ' recursion I1(n) and I2(n): xa=',xa,' xfm=',xa*amf/pim
c     scaling:
      scale=1.d0 
      fack = 1.d0
      funck0 =0.d0
      funck1 =0.d0
      funck2 =0.d0
      funck3 =0.d0

      if(iasymp.eq.0) then
      ai1mn=0.d0
      ai1n = 0.25d0*pi*expm*fderfc(-0.5d0*xlam+ratm)
      ai2mn=0.d0
      ai2n = 0.25d0*pi*expm*fderfc(+0.5d0*xlam+ratm)

      elam1= -0.5d0*dsqrt(pi)*fdexp(-0.25D0*xlam*xlam+x)/xlam
      elam2= +0.5d0*dsqrt(pi)*fdexp(-0.25D0*xlam*xlam-x)/xlam
      endif

      if(iasymp.eq.0) then
      NSTART=1
      NSTEP =1
      endif
      if(iasymp.ge.1) then
      NSTART=2
      NSTEP =2
      endif
***   DO 100 n=1,2*MAXIT           ! IF: IASYMP=0
***   DO 100 n=2,2*MAXIT,2         ! IF: IASYMP=1
      DO 100 n=NSTART,2*MAXIT,NSTEP     
c--------------------------------------------------------------
c     recurrence relation method:        ! VERY SUCCESFUL !!
c--------------------------------------------------------------
      if(iasymp.eq.0) then
      elam1 = elam1*xlam
      elam2 = elam2*xlam
      ai1pl = elam1 + 2*ratm*ai1n + 2*(n-1)*ai1mn
      ai2pl = elam2 - 2*ratm*ai2n + 2*(n-1)*ai2mn
      cfn = fdexp(-x)*ai1pl - fdexp(+x)*ai2pl
      ai1mn = ai1n
      ai1n  = ai1pl
      ai2mn = ai2n
      ai2n  = ai2pl

      as1n  = ai1pl
      as2n  = ai2pl
      endif
c--------------------------------------------------------------
c     asymptotic function:              ! LESS SUCCESFUL !!
c--------------------------------------------------------------
      if(iasymp.eq.1) then
      expmh= fdexp(0.5d0*ratm*ratm)
      psi1 = (0.25d0*pi/dsqrt(2.d0))*expmh*                    
     .fderfc(-xlam/sr2+ratm/sr2-dsqrt(1d0*n)) ! I_1: -infty - x0
      psi2 = (0.25d0*pi/dsqrt(2.d0))*expmh*                    
     .fderfc(+xlam/sr2+ratm/sr2-dsqrt(1d0*n)) ! I_2: x0 - infty
c     computation integrals:                                 
      expn = dexp(-0.5d0*n-ratm*dsqrt(2d0*n))*(2*n)**(0.5d0*n)
      as1n = (-1d0)**n*psi1*expn                                  
      as2n = psi2*expn                                  
      cfn= fdexp(-x)*as1n-fdexp(+x)*as2n
      endif     
c--------------------------------------------------------------
c     das1n = +ratp*elam0*xlam**n
c     das2n = -das1n              
c     d2as1n = +ratp**2*elam0*xlam**(n-1)*
c    .         ( (ratm-0.5d0*xlam)*xlam+n)
c     d2as2n = +ratp**2*elam0*xlam**(n-1)*
c    .         ( (ratm+0.5d0*xlam)*xlam-n)

      dcfn= -(ames/pim)*(fdexp(-x)*as1n+fdexp(+x)*as2n)
     .      +2*ratp*elam0*xlam**n
      d2cfn= +(ames/pim)**2*cfn +2*xlam**(n-1)*
     .       ratp**2*elam0*(-0.5d0*xlam**2+n)
      d3cfn= -(ames/pim)**3*(fdexp(-x)*as1n+fdexp(+x)*as2n)
     .       +2*xlam**(n-2)*ratp**3*elam0*(0.25d0*xlam**4
     .       -0.5d0*(2*n+1)*xlam**2+n*(n-1))
c---------------------------------------------------------------


c     if(icall.le.2.or.icall.eq.50.or.icall.eq.99) then
c     if(k.eq.1) write(*,*) ' xa=',xa,' xlam=',xlam 
c     write(*,700) as1n,das1n,d2as1n      
c 700 format(' as1n =',d9.2,' das1n=',d9.2,' d2as1n=',d9.2,
c    .       ' as2n =',d9.2,' das2n=',d9.2,' d2as2n=',d9.2)
c     endif

c--------------------------------------------------------------
      if(mod(n,2).eq.0) then
      k=n/2
      fack = 4*k*fack
      fct1F1 = HYP1F1(DFLOAT(-K+1),1.5D0,RATK**2)

      dfunck = fct1F1*cfn/fack   
      d1funck = fct1F1*dcfn/fack   
      d2funck = fct1F1*d2cfn/fack   
      d3funck = fct1F1*d3cfn/fack   

      funck0 = funck0 + dfunck
      funck1 = funck1 + d1funck
      funck2 = funck2 + d2funck
      funck3 = funck3 + d3funck

c     if(icall.le.2.or.icall.eq.50.or.icall.eq.99) then
c     if(k.eq.1) write(*,*) 'xfm=',x*amf/pim,' xlam=',xlam
c     write(*,20) k,dfunck,d1funck,d2funck, fct1F1
c  20 format(' k=',i3,' dfunck=',d10.3,' d1funck=',d10.3,
c    .' d2funck=',d10.3 ,' fct1F1=',d10.3)
c     endif
      endif
c--------------------------------------------------------------

  100 IF(DABS(DFUNCK/FUNCK0).LT.EPS) GOTO 200
***   WRITE(*,*) ' COFUN: XA=',XA,' ACCURACY NOT REACHED !!'
  200 FUNCO = (2/SRPI)*RATK*DEXP(-RATK*RATK)*FUNCK0
      DFUNCO = (2/SRPI)*RATK*DEXP(-RATK*RATK)*FUNCK1
      DDFUNCO = (2/SRPI)*RATK*DEXP(-RATK*RATK)*FUNCK2
      D3FUNCO  = (2/SRPI)*RATK*DEXP(-RATK*RATK)*FUNCK3
      
C     A LA SUBROUTINE FFUN2:
      X1 = 1D0/XA
      X2 = X1*X1
      X3 = X1*X2
      X4 = X2*X2
      VKUA = (ALAM/PIM)*FDEXP(-0.25D0*XLAM2)/(2*SRPI)
      DVKUA = -0.5D0*(ALAM/PIM)**2*XA*VKUA                     
      DDVKUA=-0.5D0*(ALAM/PIM)**2*(X*DVKUA+VKUA)
      FIA  = F0(EXPM,EX,DERFCM,DERFCP,XA)
      HIA  = H0(EXPM,EX,DERFCM,DERFCP,XA)
      DFIA  = -(FIA-2*VKUA)*X1 - (AMES/PIM)*HIA 
      DHIA  = -(AMES/PIM)*FIA - HIA*X1
      DDFIA = -(DFIA-2*DVKUA)*X1 +(FIA-2*VKUA)*X2 -(AMES/PIM)*DHIA
      DDHIA =-(AMES/PIM)*DFIA-DHIA*X1+HIA*X2
      D3FIA =-(DDFIA-2*DDVKUA)*X1+(DFIA-2*DVKUA)*X2-(AMES/PIM)*DDHIA
     .       +(DFIA-2*DVKUA)*X2-2*(FIA-2*VKUA)*X3
      
      F = (0.D0-FDERFC(RATK))*FIA + (2/PI)*FUNCO*X1
      DF = (0.D0-FDERFC(RATK))*DFIA + (2/PI)*DFUNCO*X1
     .       -(2/PI)*FUNCO*X2
      DDF = (0.D0-FDERFC(RATK))*DDFIA + (2/PI)*DDFUNCO*X1
     .       -(4/PI)*DFUNCO*X2 + (4/PI)*FUNCO*X3
      D3F  = (0.D0-FDERFC(RATK))*D3FIA + (2/PI)*D3FUNCO*X1
     .        -(6/PI)*DDFUNCO*X2 + (12/PI)*DFUNCO*X3
     .         -(12/PI)*FUNCO*X4
      GOTO 400
c--------------------------------------------------------------
C     PADE prediction/correction:
C     not inplemented.
c--------------------------------------------------------------
  400 ICALL = ICALL + 1
      RETURN
      END
C**************************************************************
      FUNCTION HYP1F1(A,C,Z)
C**************************************************************
C     COMPUTATION HYPERGEOMETRICAL SERIES 1F1(A,C;Z)
C**************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      FAC=1.D0
      TEMP=FAC
      AA=A
      CC=C
      DO 11 N=1,1000
           FAC=FAC*AA/CC
           FAC=FAC*Z/N
           SERIES=TEMP+FAC
           IF(SERIES.EQ.TEMP) GOTO 12
           TEMP=SERIES
           AA=AA+1.D0
           CC=CC+1.D0
   11 CONTINUE
      PAUSE 'CONVERGENCE FAILURE IN HYP1F1'
   12 HYP1F1 = SERIES
      RETURN
      END





      
        

















      
        



