C **********************************************************************
      SUBROUTINE DKBFUN(KIND,X,AM1,AM2,ALM1,ALM2,DAM,FUN)
C **********************************************************************
C
C     FOR DOUBLE STRANGENESS-EXCHANGE IN NUCLEON-NUCLEON
C     (KAON-KAON, KAON-KSTAR, KAON-KAPPA, KAON-K**)
C  ?? IT IS UNDERSTOOD THAT AM1=AMK, SO INDEX 1 REFERS TO THE KAON.
C
C     COMPUTES THE FOURIER TRANSFORMS FOR THE ADIABATIC DENOMINATOR:
C
C                     1     D  [        A            1
C      D_{//} = - --------- -- [ --------------- ---------
C                 2 OM1 OM2 DA [ (OM1+A) (OM2+A) OM1 + OM2
C
C                                         1         1     ]
C                                   + --------- --------- ] = -D_{X}
C                                     (OM1 + A) (OM2 + A) ]
C
C    ,WHICH COINCIDES WITH D_{//} FOR A=0 -> COMPATIBLE WITH PHBFUN!,
C     FOR VARIOUS COMBINATIONS OF DERIVATIVES.
C
C     KIND = 1: KAON-VECTOR-,   KIND = 2: KAON-SCALAR-,
C     KIND = 3: KAON-PSSCALAR-, KIND = 4: KAON-POMERON-EXCHANGE,
C     KIND = 5: NOT ALLOWED   , KIND = 6: KAON-AXIAL  -EXCHANGE.
C
C     SCHEMATICALLY:
C     KAON-VECTOR EXCHANGE:
C     FUN(1) = OPAEES  FUN(2) = OPAEET  FUN(3) = OPAEMT1
C     FUN(4) = OPAEMO1 FUN(5) = OPAMMC  FUN(6) = OPAMMS
C     FUN(7) = OPAMMT  FUN(8) = OPAEMS2 FUN(9) = OPAEMT2
C     FUN(10)= OPAEMO2 FUN(11)= OCREES  FUN(12)= OCREET
C     KAON-SCALAR EXCHANGE:
C     FUN(1) = OSCS    FUN(2) = OSCT    FUN(3) = OSCO
C     KAON-PSSCALAR EXCHANGE:
C     FUN(1) = OPSC    FUN(2) = OPSS    FUN(3) = OPST
C     DIFRAC-DIFRAC EXCHANGE:
C     FUN(1) = ODFC    FUN(2) = ODFO
C     KAON-AXIAL EXCHANGE:
C     FUN(1) = OAXC    FUN(2) = OAXS    FUN(3) = OAXT
C
C     FOR FORM FACTOR TREATMENT SEE:
C     TH.A.RIJKEN, ANNALS OF PHYSICS 208, P.253 (1990).
C
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/PIMAS/PIM

      DIMENSION XX(20),WW(20),YB(3),FUN(12)
      DATA PI/3.14159265D0/,SRPI/1.7724538509D0/,ICALL/0/
      DATA IPV/1/,YB/0.D0,10.D0,50.D0/,NINT/1/,NPNT/16/,IMETH/1/
      DATA AMPRO/938.2796D0/
      SAVE XX,WW
C
C     ACCORDING TO THE PAPER I, TABLE II AND APPENDIX A,B,
C     DEFINITION STATEMENT OPERATORS:
C !!  (NOTICE: G-FUNTIONS REFER TO KAON, F-FUNCTIONS TO MESONS.)
C
C 1)  KAON-VECTOR EXCHANGE:
      OPAEES(FF,D1F,D2F,D1G,D2G,D3G,XH)=
     .  2*FF*(2*D1G/XH+D2G)/3.D0

*    .+( 4*D1F*(D3G+2*D2G/XH-2*D1G/XH/XH)
*    .+(2*D1F/XH+D2F)*(2*D1G/XH+D2G) )*(PIM/AMPRO)**2/6.D0
*    .  +(1-2*IPV)*(2*D1F*D1G/XH/XH+D2F*D2G)*(PIM/AMPRO)**2/6.D0

      OPAEET(FF,D1F,D2F,D1G,D2G,D3G,XH)=
     . -2*FF*(D1G/XH-D2G)/3.D0

*    .+( 4*D1F*(D3G-  D2G/XH+  D1G/XH/XH)
*    .+(2*D1F/XH+D2F)*(D2G-D1G/XH) )*(PIM/AMPRO)**2/6.D0
*    .  +(1-2*IPV)*(D2F*D2G-D1F*D1G/XH/XH)*(PIM/AMPRO)**2/6.D0

      OCREES(FF,D1F,D2F,D1G,D2G,D3G,D4G,XH)=
     .OPAEES(FF,D1F,D2F,D1G,D2G,D3G,XH)

*    .+FF*(D4G+4*D3G/XH)*(PIM/AMPRO)**2/3.D0

      OCREET(FF,D1F,D2F,D1G,D2G,D3G,D4G,XH)=
     .OPAEET(FF,D1F,D2F,D1G,D2G,D3G,XH)

*    .+FF*(D4G+D3G/XH-6*D2G/XH**2+6*D1G/XH**3)*(PIM/AMPRO)**2/3.D0

      OPAEMT(D1F,D1G,D2G,XH)=  2*D1F*(D1G/XH-D2G)/XH
      OPAEMO(D1F,D1G,XH)=  4*D1F*D1G/XH/XH
      OPEMS2(D1F,D2F,D1G,D2G,XH)= (2*D1F/XH+D2F)*(2*D1G/XH+D2G)/3.D0
      OPEMT2(D1F,D2F,D1G,D2G,XH)=  (D1F/XH-D2F)*(D1G/XH-D2G)/3.D0
      OPEMO2(D1F,D1G,XH)=  2*D1F*D1G/XH/XH

      OPAMMC(D1F,D2F,D1G,D2G,XH)=
     .2*( D1F*D1G/XH + D1F*D2G + D2F*D1G)/XH
      OPAMMS(D1F,D2F,D1G,D2G,XH)=
     .-2*( 3*D1F*D1G/XH/XH + D1F*D2G/XH + D2F*D1G/XH +D2F*D2G)/3.D0
      OPAMMT(D1F,D2F,D1G,D2G,XH)=
     .+(D2F+D1F/XH)*(D2G-D1G/XH)/3.D0-1*(D2F-D1F/XH)*D1G/XH/3.D0
     .-1*(D2G-D1G/XH)*D1F/XH/3.D0

C2,4) KAON-SCALAR EXCHANGE:
      OSCS(FF,D1F,D2F,D1G,D2G,XH)= -2*FF*(2*D1G/XH+D2G)/3.D0

*    .  +(2*D1F/XH+D2F)*(2*D1G/XH+D2G)*(PIM/AMPRO)**2/6.D0
*    .  -(1-2*IPV)*(2*D1F*D1G/XH/XH+D2F*D2G)*(PIM/AMPRO)**2/6.D0
      OSCT(FF,D1F,D2F,D1G,D2G,XH)= +2*FF*(D1G/XH-D2G)/3.D0

*    .      +(D1F/XH-D2F)*(D1G/XH-D2G)*(PIM/AMPRO)**2/6.D0
*    .  +(1-2*IPV)*(D1F*D1G/XH/XH-D2F*D2G)*(PIM/AMPRO)**2/6.D0
      OSCO(D1F,D1G,XH)= 0.D0
*    .                 +(D1F*D1G/XH/XH)*(PIM/AMPRO)**2

C 3)  KAON-PSEUDO-SCALAR EXCHANGE:
      OPSC(D1F,D2F,D1G,D2G,XH)= 2*(2*D1F*D1G/XH/XH+D2F*D2G)
      OPSS(D1F,D2F,D1G,D2G,XH)=+4*(D1F*D1G/XH+D1F*D2G+D2F*D1G)/XH/3.D0
      OPST(D1F,D2F,D1G,D2G,XH)=-2*( (D2F-D1F/XH)*D1G/XH +
     . (D2G-D1G/XH)*D1F/XH )/3.D0

C 4)  DIFRAC-DIFRAC EXCHANGE:
      ODFC(FF,D1F,D2F,GG,D1G,D2G,XH)= 2*FF*GG

*    . -0.5D0*(PIM/AMPRO)**2
*    . *((D2F+2*D1F/XH)*GG+FF*(D2G+2*D1G/XH)-2*D1F*D1G)
      ODFO(FF,D1F,GG,D1G,XH)= 0.D0

*    .                        -(D1F*GG+FF*D1G)/XH*(PIM/AMPRO)**2

C 5)  KAON-AXIAL EXCHANGE:
      OAXC(FF,D1G,D2G,XH)= -2*FF*(2*D1G/XH+D2G)
      OAXS(FF,D1G,D2G,XH)= +2*FF*(2*D1G/XH+D2G)*2/3.D0
      OAXT(FF,D1G,D2G,XH)= -2*FF*( -D1G/XH+D2G)/3.D0
C
C     POTENTIAL FUNTION DEFINITION
C
      F0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH-EPH*EXH )/(2*XH)
      H0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH+EPH*EXH )/(2*XH)
C
      CALL ERRSET(208,256,-1,1)
      IF(ICALL.EQ.0) THEN
        CALL GAUSS(NPNT,XX,WW,1)
        IF(IMETH.EQ.1) NINT=1
C       PRINT 52, IMETH,NPNT
C52     FORMAT(//,'  IN HFUN: IMETH=',I2,' NPNT=',I2,//)
        ICALL=1
      ENDIF
CC
      AA   = DAM/PIM
      AA2  = AA*AA
      X1   = 1.D0/X
      X2   = X1*X1
      X3   = X1*X2
      X4   = X1*X3
CC
      RATP1=PIM/ALM1
      XLM1 =0.5D0*X*ALM1/PIM
      XLM12=XLM1*XLM1
      VKU1 =(ALM1/PIM)*FDEXP(-XLM12)/(2*SRPI)
      DVKU1=-0.5D0*(ALM1/PIM)**2*X*VKU1
      DDVKU1=-0.5D0*(ALM1/PIM)**2*(VKU1+X*DVKU1)
      D3VKU1=-0.5D0*(ALM1/PIM)**2*(2*DVKU1+X*DDVKU1)

      RATP2=PIM/ALM2
      XLM2 =0.5D0*X*ALM2/PIM
      XLM22=XLM2*XLM2
      VKU2 =(ALM2/PIM)*FDEXP(-XLM22)/(2*SRPI)
      DVKU2=-0.5D0*(ALM2/PIM)**2*X*VKU2
C
C     CONSTRUCTION BASE FUNCTIONS BY INTEGRATION
C
      DO 105 IFUN=1,12
  105 FUN(IFUN) = 0.D0
      DO 100 INT=1,NINT
      ABM=0.5D0*(YB(INT+1)-YB(INT))
      ABP=0.5D0*(YB(INT+1)+YB(INT))
      DO 110 IY=1,NPNT
      IF(IMETH.EQ.0) THEN
      Y=ABM*XX(IY)+ABP
      GEW=WW(IY)*ABM*1.D0/PI
      ENDIF
      IF(IMETH.EQ.1) THEN
      Y=(1.D0+XX(IY))/(1.D0-XX(IY))
      GEW=WW(IY)*(2.D0/(1.D0-XX(IY))**2)*1.D0/PI
      ENDIF
      IF(Y.GT.50.D0) GOTO 110 
C     FACTOR FROM FORM FACTOR EFFECTS THROUGH DISP. REL. :
      FDISP1=FDEXP(-Y*Y*RATP1*RATP1)
      FDISP2=FDEXP(-Y*Y*RATP2*RATP2)
      FDISP = FDISP1*FDISP2
C
      IF(KIND.NE.5) THEN
      AM=PIM*DSQRT((AM1/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM1
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM1+RATM)
      EPM  =FDERFC( XLM1+RATM)
C
      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU1)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU1)*X1+(FIM-2*VKU1)*X2-(AM/PIM)*DHIM
      DDHIM  =-(AM/PIM)*DFIM -DHIM*X1 +HIM*X2
      D3FIM  =-(DDFIM-2*DDVKU1)*X1+2*(DFIM-2*DVKU1)*X2
     .        -2*(FIM-2*VKU1)*X3-(AM/PIM)*DDHIM
      D3HIM  =-(AM/PIM)*DDFIM -DDHIM*X1 +2*DHIM*X2-2*HIM*X3
      D4FIM  =-(D3FIM-2*D3VKU1)*X1+3*(DDFIM-2*DDVKU1)*X2
     .        -6*(DFIM-2*DVKU1)*X3+6*(FIM-2*VKU1)*X4-(AM/PIM)*D3HIM

      FIM1  = FIM
      DFIM1  = DFIM
      DDFIM1  = DDFIM
      D3FIM1  = D3FIM
      D4FIM1  = D4FIM
      ENDIF
C
      IF(KIND.LT.4.OR.KIND.GE.6) THEN
      AM=PIM*DSQRT((AM2/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM2
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM2+RATM)
      EPM  =FDERFC( XLM2+RATM)

      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU2)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU2)*X1+(FIM-2*VKU2)*X2-(AM/PIM)*DHIM
      FIM2  = FIM
      DFIM2  = DFIM
      DDFIM2  = DDFIM
      ENDIF
      IF(KIND.EQ.4.OR.KIND.EQ.5) THEN
      XA   =X*AM2/PIM
      FIA  =(4.D0/SRPI)*(AM2/PIM)*(AM2/AMPRO)**2*FDEXP(-XA*XA)
      DFIA  = -2*(AM2/PIM)*XA*FIA
      DDFIA  = -2*(AM2/PIM)*(XA*DFIA + FIA*AM2/PIM)
      FIM2= FIA
      DFIM2= DFIA
      DDFIM2= DDFIA
      ENDIF

C
      WGHT= GEW*(AA2-Y*Y)/(AA2+Y*Y)**2
C
      IF(KIND.EQ.1) THEN
      FUN(1)  = FUN(1)  + WGHT*FDISP*
     .  OPAEES(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,D3FIM1,X)
      FUN(2)  = FUN(2)  + WGHT*FDISP*
     .  OPAEET(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,D3FIM1,X)
      FUN(11) = FUN(11) + WGHT*FDISP*
     .   OCREES(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,D3FIM1,D4FIM1,X)
      FUN(12) = FUN(12) + WGHT*FDISP*
     .   OCREET(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,D3FIM1,D4FIM1,X)
      FUN(3)  = FUN(3)  + WGHT*FDISP*
     .   OPAEMT(DFIM2,DFIM1,DDFIM1,X)
      FUN(4)  = FUN(4)  + WGHT*FDISP*
     .   OPAEMO(DFIM2,DFIM1,X)
      FUN(5)  = FUN(5)  + WGHT*FDISP*
     .   OPAMMC(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(6)  = FUN(6)  + WGHT*FDISP*
     .   OPAMMS(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(7)  = FUN(7)  + WGHT*FDISP*
     .   OPAMMT(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(8)  = FUN(8)  + WGHT*FDISP*
     .   OPEMS2(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(9)  = FUN(9)  + WGHT*FDISP*
     .   OPEMT2(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(10) = FUN(10)  + WGHT*FDISP*
     .   OPEMO2(DFIM2,DFIM1,X)
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1)  = FUN(1)  + WGHT*
     .          FDISP*OSCS(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(2)  = FUN(2)  + WGHT*
     .          FDISP*OSCT(FIM2,DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(3)  = FUN(3)  + WGHT*
     .          FDISP*OSCO(DFIM2,DFIM1,X)
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1)  = FUN(1)  + WGHT*
     .          FDISP*OPSC(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(2)  = FUN(2)  + WGHT*
     .          FDISP*OPSS(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      FUN(3)  = FUN(3)  + WGHT*
     .          FDISP*OPST(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
      ENDIF
C
C     IF(KIND.EQ.5) THEN: NOT ALLOWED!
C
      IF(KIND.EQ.6) THEN
      FUN(1)  = FUN(1)  + WGHT*
     .          FDISP*OAXC(FIM2,DFIM1,DDFIM1,X)
      FUN(2)  = FUN(2)  + WGHT*
     .          FDISP*OAXS(FIM2,DFIM1,DDFIM1,X)
      FUN(3)  = FUN(3)  + WGHT*
     .          FDISP*OAXT(FIM2,DFIM1,DDFIM1,X)
      ENDIF
C
110   CONTINUE
100   CONTINUE

CC
C     PRINT 99, KIND,X,AM1,AM2,ALM1,ALM2,(FUN(II),II=1,12)
C99   FORMAT(/,' IN DKBFUN: KIND=',I2,' X=',F10.3,/,
C    .'  AM1,AM2=',2(F10.3,3X),'  ALM1,ALM2=',2(F10.3,3X),/,
C    .       /,' IN DKBFUN: FUN(II)=',/,5(D10.3,3X),/)
CC
1000  CALL ERRSET(208,256,1,1)
      RETURN
      END
C **********************************************************************
      SUBROUTINE DKBOOM(ITYP,X,AM1,AM2,ALM1,ALM2,DAM)
C **********************************************************************
C
C     FOR DOUBLE STRANGENESS EXCHANGE (KK, K-KSTAR, K-KAPPA, K-K**)
C     COMPUTATION 1/M-CONTRIBUTIONS FOR TWO-MESON EXCHANGE
C     NOTE: PARTICLE 1 IS THE KAON!
C
C     ITYP=1: PS-PS, ITYP=2: PS-VCT, ITYP=3: PS-SCA, ITYP=4: PS-DIF
C     WARNING: ITYP=5 NOT YET IMPLEMENTED!
C *********************************************************************
C
C     NON-ADIABATIC DENOMINATOR (A LA EQ. (5.1)):
C                         1    D^2  [        A            1
C       D(//)^(1)=   --------- ---- [ --------------- ---------
C                    4 OM1 OM2 DA^2 [ (OM1+A) (OM2+A) OM1 + OM2
C
C                                 1         1     ]
C                           + --------- --------- ] = - D(X)^(1)/2
C                             (OM1 + A) (OM2 + A) ]
C ---------------------------------------------------------------------
C
C     PV-VERTEX CORRECTION DENOMINATOR (A LA EQ. (5.6)):
C                              1          [        2A     ]
C       D(PV)^(1)=  -------------------   [ 1 + --------- ]
C                   2 (OM1+A)^2 (OM2+A)^2 [     OM1 + OM2 ]
C
C     WHICH COINCIDE WITH FUNCTIONS COMPUTED BY FOOM FOR A-> 0!
C
C     A = DAM/PIM = HYPERON-NUCLEON MASS DIFFERENCE
C
C     WE COMPUTE IN THIS ROUTINE:
C
C  1) ITYPE=1 (PS-PS):
C  1A FNONC: -F.T. (K1.K2)^3 D//^(1)
C     FNONS: -F.T. (K1.K2) SIG1.(K1XK2) SIG2.(K1XK2) D(//)^(1)
C     FNONT: -F.T. (K1.K2) SIG1.(K1XK2) SIG2.(K1XK2) D(//)^(1)
C     FNONO: +F.T. (K1.K2) i*(SIG1+SIG2).(K1XK2) Q.(K1-K2) D(//)^(1)
C  1B FPVC : +F.T. (K1.K2) (K1^2+K2^2) D(PV)^(1)
C     FPVO : +F.T. i*(SIG1+SIG2).[QX ......] D(PV)^(1)
C
C  2) ITYPE=2,3,4 (PS-VCT,PS-SCAL,PS-DIFF):
C  2A FNONS,FNONT: +F.T. (K1.K2) SIG1.K1 SIG2.K1 D(//)^(1)
C  2B FPVS ,FPVT : -F.T. (SIG1.K1 SIG2.K2+SIG1.K2 SIG2.K1) D(PV)^(1)
C
C     FOR FORM FACTOR TREATMENT SEE:
C     TH.A.RIJKEN, ANNALS OF PHYSICS 208, P.253 (1990).
C
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/PIMAS/PIM
      COMMON/FOOMS/FNONC,FNONS,FNONT,FNONO,FPVC,FPVS,FPVT,FPVO
     .            ,FOFFC,FOFFS,FOFFT,FOFFO,FOFFS2,FOFFT2
      DIMENSION XX(20),WW(20),YB(3)
      DATA PI/3.14159265D0/,SRPI/1.7724538509D0/,ICALL/0/
      DATA YB/0.D0,10.D0,50.D0/,NINT/1/,NPNT/16/,IMETH/0/
      SAVE XX,WW,AA,AA2

C
C     CONSTRUCTION BASE FUNCTIONS FOR PS-PS-EXCHANGE:
C
C     ITYP=1:
      FNONC1(D1F,D2F,D3F,D1G,D2G,D3G,XH) = +0.5D0*(
     .  6*(D2F-D1F/XH)*(D2G-D1G/XH)/XH/XH+D3F*D3G)
      FNONS1(D1F,D2F,D3F,D1G,D2G,D3G,XH) = -(1.D0/3.D0)*(
     .  (D2F-D1F/XH-XH*D3F)*(D2G-D1G/XH-XH*D3G)/XH/XH-D3F*D3G)
      FNONT1(D1F,D2F,D3F,D1G,D2G,D3G,XH) = +(2.D0/3.D0)*(
     .  (D2F-D1F/XH-XH*D3F/4)*(D2G-D1G/XH-XH*D3G/4)/XH/XH
     .  -D3F*D3G/16.D0)
      FNONO1(D1F,D2F,D1G,D2G,XH) = -2*
     .  (D2F-D1F/XH)*(D2G-D1G/XH)/XH/XH
      FPVC1(D1F,D2F,D3F,D1G,XH) =
     .  (-2*D1F/XH/XH+2*D2F/XH+D3F)*D1G
      FPVO1(D1F,D2F,D1G,D2G,XH) =
     .  2*(D1F*D2G+D2F*D1G)/XH+4*D1F*D1G/XH/XH

C     ITYP=2,3,4:
      FNONS2(D1F,D2F,D3F,D1G,XH) = +(1.D0/6.D0)*
     .  (D3F+2*D2F/XH-2*D1F/XH/XH)*D1G
      FNONT2(D1F,D2F,D3F,D1G,XH) = +(1.D0/6.D0)*
     .  (D3F-  D2F/XH+  D1F/XH/XH)*D1G
      FPVS2(D1F,D1G,XH) = +D1F*D1G/3.D0
      FPVT2(D1F,D1G,XH) = +D1F*D1G/3.D0
C     ITYP=5:
      FNONC5(D1F,D1G,XH) = +0.5D0*D1F*D1G
C
C     POTENTIAL FUNTION DEFINITION
C
      F0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH-EPH*EXH )/(2*XH)
      H0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH+EPH*EXH )/(2*XH)
C
      CALL ERRSET(208,256,-1,1)
        AA   =DAM/PIM
      IF(ICALL.EQ.0) THEN
        AA2  =AA*AA
        CALL GAUSS(NPNT,XX,WW,1)
        IF(IMETH.EQ.1) NINT=1
C       PRINT 52, IMETH,NPNT
C52     FORMAT(//,'  IN HFUN: IMETH=',I2,' NPNT=',I2,//)
        ICALL=1
      ENDIF
CC
      X1   = 1.D0/X
      X2   = X1*X1
      X3   = X1*X2
CC
      RATP1=PIM/ALM1
      XLM1 =0.5D0*X*ALM1/PIM
      XLM12=XLM1*XLM1
      VKU1 =(ALM1/PIM)*FDEXP(-XLM12)/(2*SRPI)
      DVKU1=-0.5D0*(ALM1/PIM)**2*X*VKU1
      DDVKU1=-0.5D0*(ALM1/PIM)**2*(VKU1+X*DVKU1)

      RATP2=PIM/ALM2
      XLM2 =0.5D0*X*ALM2/PIM
      XLM22=XLM2*XLM2
      VKU2 =(ALM2/PIM)*FDEXP(-XLM22)/(2*SRPI)
      DVKU2=-0.5D0*(ALM2/PIM)**2*X*VKU2
      DDVKU2=-0.5D0*(ALM2/PIM)**2*(VKU2+X*DVKU2)
C
C     CONSTRUCTION BASE FUNCTIONS BY INTEGRATION
C
      FNONC=0.D0
      FNONS=0.D0
      FNONT=0.D0
      FNONO=0.D0
      FPVC =0.D0
      FPVS =0.D0
      FPVT =0.D0
      FPVO =0.D0
      DO 100 INT=1,NINT
      ABM=0.5D0*(YB(INT+1)-YB(INT))
      ABP=0.5D0*(YB(INT+1)+YB(INT))
      DO 110 IY=1,NPNT
      IF(IMETH.EQ.0) THEN
      Y=ABM*XX(IY)+ABP
      GEW=WW(IY)*ABM*2.D0/PI
      ENDIF
      IF(IMETH.EQ.1) THEN
      Y=(1.D0+XX(IY))/(1.D0-XX(IY))
      GEW=WW(IY)*(2.D0/(1.D0-XX(IY))**2)*2.D0/PI
      ENDIF
C     FACTOR FROM FORM FACTOR EFFECTS THROUGH DISP. REL. :
      FDISP1=FDEXP(-Y*Y*RATP1*RATP1)
      FDISP2=FDEXP(-Y*Y*RATP2*RATP2)
      FDISP = FDISP1*FDISP2
      GEW = GEW*FDISP
C
      AM=PIM*DSQRT((AM1/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM1
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM1+RATM)
      EPM  =FDERFC( XLM1+RATM)
C
      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU1)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU1)*X1+(FIM-2*VKU1)*X2-(AM/PIM)*DHIM

      DDHIM =-(AM/PIM)*DFIM-DHIM*X1+HIM*X2
      D3FIM =+(DFIM-2* DVKU1)*X2-2*(FIM-2* VKU1)*X3-(AM/PIM)*DDHIM
     .       -(DDFIM-2*DDVKU1)*X1+  (DFIM-2*DVKU1)*X2

      FIM1  = FIM
      DFIM1  = DFIM
      DDFIM1  = DDFIM
      D3FIM1   = D3FIM
C
      AM=PIM*DSQRT((AM2/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM2
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM2+RATM)
      EPM  =FDERFC( XLM2+RATM)

      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU2)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU2)*X1+(FIM-2*VKU2)*X2-(AM/PIM)*DHIM

      DDHIM =-(AM/PIM)*DFIM-DHIM*X1+HIM*X2
      D3FIM =+(DFIM-2* DVKU2)*X2-2*(FIM-2* VKU2)*X3-(AM/PIM)*DDHIM
     .       -(DDFIM-2*DDVKU2)*X1+  (DFIM-2*DVKU2)*X2

      FIM2  = FIM
      DFIM2  = DFIM
      DDFIM2  = DDFIM
      D3FIM2   = D3FIM
C
C     NON-ADIABATIC INTEGRALS:
      WGHT= GEW*AA*(AA2-3*Y**2)/(Y*Y+AA2)**3
      IF(ITYP.EQ.1) THEN
      FNONC = FNONC + WGHT*
     .        FNONC1(DFIM1,DDFIM1,D3FIM1,DFIM2,DDFIM2,D3FIM2,X)
      FNONS = FNONS + WGHT*
     .        FNONS1(DFIM1,DDFIM1,D3FIM1,DFIM2,DDFIM2,D3FIM2,X)
      FNONT = FNONT + WGHT*
     .        FNONT1(DFIM1,DDFIM1,D3FIM1,DFIM2,DDFIM2,D3FIM2,X)
      FNONO = FNONO + WGHT*
     .        FNONO1(DFIM1,DDFIM1,DFIM2,DDFIM2,X)
      ENDIF
      IF(ITYP.GE.2 .AND. ITYP.LE.4) THEN
      FNONS = FNONS + WGHT*
     .        FNONS2(DFIM1,DDFIM1,D3FIM1,DFIM2,X)
      FNONT = FNONT + WGHT*
     .        FNONT2(DFIM1,DDFIM1,D3FIM1,DFIM2,X)
      ENDIF
      IF(ITYP.EQ.5) THEN
      FNONC = FNONC + WGHT*FNONC5(DFIM1,DFIM2,X)
      ENDIF

C     PV-VERTEX CORRECTION INTEGRALS:
      WGHT= GEW*2*AA*Y**2/(Y*Y+AA2)**2
      IF(ITYP.EQ.1) THEN
      FPVC  = FPVC  + WGHT*(
     .        FPVC1(DFIM1,DDFIM1,D3FIM1,DFIM2,X) +
     .        FPVC1(DFIM2,DDFIM2,D3FIM2,DFIM1,X) )
      FPVO  = FPVO  + WGHT*
     .        FPVO1(DFIM1,DDFIM1,DFIM2,DDFIM2,X)
      ENDIF
      IF(ITYP.GE.2 .AND. ITYP.LE.4) THEN
      FPVS  = FPVS  + WGHT*FPVS2(DFIM1,DFIM2,X)
      FPVT  = FPVT  + WGHT*FPVT2(DFIM1,DFIM2,X)
      ENDIF
C
110   CONTINUE
100   CONTINUE

CC
10000 CALL ERRSET(208,256,1,1)
      RETURN
      END


C **********************************************************************
      SUBROUTINE YNFUN2(KIND,X,AM1,AM2,ALM1,ALM2,DAMB,IYEX,FUN)
C
C   THIS ROUTINE IS FOR THE CASE OF ONE(!) MESON CARRYING STRANGENESS,
C   THIS IS EITHER MESON1(IYEX=1) OR MESON2(IYEX=2).
C   VERSION: 23 FEBRUARI, 1996
C **********************************************************************
C
C     THIS ROUTINE IS CALLED FROM: YNPIBE.FORTRAN ; YNPSBE.FORTRAN
C     IT IS UNDERSTOOD THAT INDEX 1 REFERS TO THE PS-EXCHANGE (PION ETC.
C
C     COMPUTES THE FOURIER TRANSFORMS OF D, WHERE
C
C               1     |      1           1     |    1
C   2*D =  ---------- |  --------- + --------- |. -----
C          2 OM1.OM2  |  OM1+OM2-A   OM1+OM2+A |  OM1^2
C
C            1   (OM2^2 + A^2)      1
C        + ----- ---------------. -----  , D_{//}=-D_{X} = D
C           OM2  (OM2^2 + A^2)^2  OM1^2
C
C     FOR VARIOUS COMBINATIONS OF DERIVATIVES.
C
C     NOTICE: THIS CORRESPONDS TO YNFUN(....) FOR THE LIMIT DAM=0.D0!
C
C     KIND = 1: PION-VECTOR-,   KIND = 2: PION-SCALAR-,
C     KIND = 3: PION-PSSCALAR-, KIND = 4: PION-POMERON-EXCHANGE.
C
C     SCHEMATICALLY:
C     PION-VECTOR EXCHANGE:
C     FUN(1) = OPAEES  FUN(2) = OPAEET  FUN(3) = OPAEMT1
C     FUN(4) = OPAEMO  FUN(5) = OPAMMC  FUN(6) = OPAMMS
C     FUN(7) = OPAMMT  FUN(8) = OPAEMS2 FUN(9) = OPAEMT2
C     FUN(10)= OPAEMO2 FUN(11)= OCREES  FUN(12)= OCREET 
C     PION-SCALAR EXCHANGE:
C     FUN(1) = OSCS    FUN(2) = OSCT
C     PION-PSSCALAR EXCHANGE:
C     FUN(1) = OPSC    FUN(2) = OPSS    FUN(3) = OPST
C  !! NOTE: simplest def. of functions here, no k^2/M^2-terms!
C           in OPAEES etc.
C
C     FOR FORM FACTOR TREATMENT SEE:
C     TH.A.RIJKEN, ANNALS OF PHYSICS 208, P.253 (1990).
C
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
***   COMPLEX*16 CDSQRT,DREAL,DIMAG
***   COMMON/MASSES/PIM,AMNC(3),AMY(3),REDM(3),PLAB,AK(3),AKS(3),TEM(3)
      DIMENSION XX(20),WW(20),YB(3),FUN(12)
      DATA PI/3.14159265D0/,SRPI/1.7724538509D0/,ICALL/0/
      DATA YB/0.D0,10.D0,50.D0/,NINT/1/,NPNT/16/,IMETH/0/
      DATA PIM/138.041D0/,AMPRO/938.2796D0/
      DATA ZI/(0.D0,1.D0)/
      SAVE XX,WW,DAM
C
C     ACCORDING TO THE PAPER APPENDIX B,
C     DEFINITION STATEMENT OPERATORS:
C
C 1)  PION-VECTOR EXCHANGE:
      OPAEES(FF,D1G,D2G,XH)=  2*FF*(2*D1G/XH+D2G)/3.D0
      OPAEET(FF,D1G,D2G,XH)= -2*FF*(D1G/XH-D2G)/3.D0
      OCREES(FF,D1G,D2G,XH)=
     .OPAEES(FF,D1G,D2G,XH)
      OCREET(FF,D1G,D2G,XH)=
     .OPAEET(FF,D1G,D2G,XH)

      OPAEMT(D1F,D1G,D2G,XH)=  2*D1F*(D1G/XH-D2G)/XH
      OPAEMO(D1F,D1G,XH)=  4*D1F*D1G/XH/XH
      OPEMS2(D1F,D2F,D1G,D2G,XH)=(2*D1F/XH+D2F)*(2*D1G/XH+D2G)/3.D0
      OPEMT2(D1F,D2F,D1G,D2G,XH)=(D1F/XH-D2F)*(D1G/XH-D2G)/3.D0
      OPEMO2(D1F,D1G,XH)= 2*D1F*D1G/XH/XH               

      OPAMMC(D1F,D2F,D1G,D2G,XH)=
     .2*( D1F*D1G/XH + D1F*D2G + D2F*D1G)/XH
      OPAMMS(D1F,D2F,D1G,D2G,XH)=
     .-2*( 3*D1F*D1G/XH/XH + D1F*D2G/XH + D2F*D1G/XH +D2F*D2G)/3.D0
      OPAMMT(D1F,D2F,D1G,D2G,XH)=
     .-(D1F/XH+D2F)*(D1G/XH-D2G)/3.D0+2*(D1F/XH-D2F)*D1G/XH/3.D0

C 2)  PION-SCALAR EXCHANGE:
      OSCS(FF,D1G,D2G,XH)= -2*FF*(2*D1G/XH+D2G)/3.D0
      OSCT(FF,D1G,D2G,XH)= +2*FF*(D1G/XH-D2G)/3.D0
      OSCO(D1F,D1G,XH)= 0.D0

C 3)  PION-PSEUDO-SCALAR EXCHANGE:
      OPSC(D1F,D2F,D1G,D2G,XH)= 2*(2*D1F*D1G/XH/XH+D2F*D2G)
      OPSS(D1F,D2F,D1G,D2G,XH)= 4*(D1F*D1G/XH+D1F*D2G+D2F*D1G)/XH/3.D0
      OPST(D1F,D2F,D1G,D2G,XH)= 2*( (D1F/XH-D2F)*D1G/XH +
     . (D1G/XH-D2G)*D1F/XH )/3.D0
C
C     POTENTIAL FUNTION DEFINITION
C
      F0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH-EPH*EXH )/(2*XH)
      H0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH+EPH*EXH )/(2*XH)
      ZF0(ZERATH,ZEXH,ZEMH,ZEPH,XH)=ZERATH*(ZEMH/ZEXH-ZEPH*ZEXH)/(2*XH)
      ZH0(ZERATH,ZEXH,ZEMH,ZEPH,XH)=ZERATH*(ZEMH/ZEXH+ZEPH*ZEXH)/(2*XH)
C
      CALL ERRSET(208,256,-1,1)
      IF(ICALL.EQ.0) THEN
        CALL GAUSS(NPNT,XX,WW,1)
        IF(IMETH.EQ.1) NINT=1
C       PRINT 52, IMETH,NPNT
C52     FORMAT(//,'  IN HFUN: IMETH=',I2,' NPNT=',I2,//)
        DAM = DAMB/PIM
        ICALL=1
      ENDIF
CC
      X1   = 1.D0/X
      X2   = X1*X1
CC
      DAMB1 = DAM*(2-IYEX)
      DAMB2 = DAM*(IYEX-1)
      IF(IYEX.EQ.1) FDISA=FDEXP(DAMB**2/ALM1**2)
      IF(IYEX.EQ.2) FDISA=FDEXP(DAMB**2/ALM2**2)

      RATP1=PIM/ALM1
      XLM1 =0.5D0*X*ALM1/PIM
      XLM12=XLM1*XLM1
      VKU1 =(ALM1/PIM)*FDEXP(-XLM12)/(2*SRPI)
      DVKU1=-0.5D0*(ALM1/PIM)**2*X*VKU1

      RATP2=PIM/ALM2
      XLM2 =0.5D0*X*ALM2/PIM
      XLM22=XLM2*XLM2
      VKU2 =(ALM2/PIM)*FDEXP(-XLM22)/(2*SRPI)
      DVKU2=-0.5D0*(ALM2/PIM)**2*X*VKU2

      BM1 = DSQRT(AM1**2-DAMB1**2)
      XA   =X*BM1/PIM
      RATA =BM1/ALM1
      ERATA=DEXP(RATA*RATA)
      EXA  =DEXP(XA)
      EMA  =FDERFC(-XLM1 +RATA)
      EPA  =FDERFC( XLM1 +RATA)
      FIA  =F0(ERATA,EXA ,EMA ,EPA ,X)
      HIA  =H0(ERATA,EXA ,EMA ,EPA ,X)
      DFIA =-(FIA -2*VKU1)*X1-(BM1/PIM)*HIA
      DHIA  =-(BM1/PIM)*FIA -HIA *X1
      DDFIA  =-(DFIA-2*DVKU1)*X1+(FIA-2*VKU1)*X2-(BM1/PIM)*DHIA
      FI1= FIA
      DFI1= DFIA
      DDFI1= DDFIA

      IF(KIND.NE.4) THEN
      BM2 = DSQRT(AM2**2-DAMB2**2)
      XA   =X*BM2/PIM
      RATA =BM2/ALM2
      ERATA=DEXP(RATA*RATA)
      EXA  =DEXP(XA)
      EMA  =FDERFC(-XLM2 +RATA)
      EPA  =FDERFC( XLM2 +RATA)
      FIA  =F0(ERATA,EXA ,EMA ,EPA ,X)
      HIA  =H0(ERATA,EXA ,EMA ,EPA ,X)
      DFIA =-(FIA -2*VKU2)*X1-(BM2/PIM)*HIA
      DHIA  =-(BM2/PIM)*FIA -HIA *X1
      DDFIA  =-(DFIA-2*DVKU2)*X1+(FIA-2*VKU2)*X2-(BM2/PIM)*DHIA
      DDHIA =-(BM2/PIM)*DFIA-DHIA*X1+HIA*X2
C     D3FIA =-(DDFIA-2*DDVKUA)*X1+(DFIA-2*DVKUA)*X2-(BM2/PIM)*DDHIA
C    .       +(DFIA-2*DVKUA)*X2-2*(FIA-2*VKUA)*X3
C     D3HIA =-(BM2/PIM)*DDFIA-DDHIA*X1+2*DHIA*X2-2*HIA*X3

      F4A   = 0.5D0*(PIM/BM2)*X*HIA
      DF4A  = 0.5D0*(PIM/BM2)*(HIA+X*DHIA)
      DDF4A = 0.5D0*(PIM/BM2)*(2*DHIA+X*DDHIA)
C     D3F4A  = 0.5D0*(PIM/BM2)*(3*DDHIA+X*D3HIA)
      ENDIF
      IF(KIND.EQ.4) THEN
      XA   =X*AM2/PIM
      FIA  =(4.D0/SRPI)*(AM2/PIM)*(AM2/AMPRO)**2*FDEXP(-XA*XA)
      DFIA  = -2*(AM2/PIM)*XA*FIA
      DDFIA = -2*(AM2/PIM)*(XA*DFIA + DFIA*AM2/PIM)
C     D3FIA = -2*(AM2/PIM)*(XA*DDFIA + 2*DFIA*AM2/PIM)
      RATD  = 0.5D0*PIM/AM2
      F4A  = FIA*RATD**2
      DF4A = DFIA*RATD**2
      DDF4A= DDFIA*RATD**2
C     D3F4A= D3FIA*RATD**2
      ENDIF
      FI2= FIA
      DFI2= DFIA
      DDFI2= DDFIA
      FI42= F4A
      DFI42= DF4A
      DDFI42= DDF4A
C
C     CONSTRUCTION BASE FUNCTIONS BY INTEGRATION
C
      DO 105 IFUN=1,12
  105 FUN(IFUN) = 0.D0
      IF(KIND.EQ.1) THEN
      FUN(1) = 2*DAM*OPAEES(FI42,DFI1,DDFI1,X)*FDISA
      FUN(11)= 2*DAM*OCREES(FI42,DFI1,DDFI1,X)*FDISA
      FUN(2) = 2*DAM*OPAEET(FI42,DFI1,DDFI1,X)*FDISA
      FUN(12)= 2*DAM*OCREET(FI42,DFI1,DDFI1,X)*FDISA
      FUN(3) = 2*DAM*OPAEMT(DFI42,DFI1,DDFI1,X)*FDISA
      FUN(4) = 2*DAM*OPAEMO(DFI42,DFI1,X)*FDISA
      FUN(5) = 2*DAM*OPAMMC(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(6) = 2*DAM*OPAMMS(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(7) = 2*DAM*OPAMMT(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(8) = 2*DAM*OPEMS2(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(9) = 2*DAM*OPEMT2(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(10)= 2*DAM*OPEMO2(DFI42,DFI1,X)*FDISA
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1) = 2*DAM*OSCS(FI42,DFI1,DDFI1,X)*FDISA
      FUN(2) = 2*DAM*OSCT(FI42,DFI1,DDFI1,X)*FDISA
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1) = 2*DAM*OPSC(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(2) = 2*DAM*OPSS(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      FUN(3) = 2*DAM*OPST(DFI42,DDFI42,DFI1,DDFI1,X)*FDISA
      ENDIF
      DO 100 INT=1,NINT
      ABM=0.5D0*(YB(INT+1)-YB(INT))
      ABP=0.5D0*(YB(INT+1)+YB(INT))
      DO 110 IY=1,NPNT
      IF(IMETH.EQ.0) THEN
      Y=ABM*XX(IY)+ABP
      GEW=WW(IY)*ABM*1.D0/PI
      ENDIF
      IF(IMETH.EQ.1) THEN
      Y=(1.D0+XX(IY))/(1.D0-XX(IY))
      GEW=WW(IY)*(2.D0/(1.D0-XX(IY))**2)*1.D0/PI
      ENDIF
      IF(Y.GT.50.D0) GOTO 110 
C     FACTOR FROM FORM FACTOR EFFECTS THROUGH DISP. REL. :
      FDISP1=FDEXP(-Y*Y*RATP1*RATP1)
      FDISP2=FDEXP(-Y*Y*RATP2*RATP2)
      FDISP = FDISP1*FDISP2
      ZDISP1= CDEXP(2*ZI*Y*DAM*RATP1**2)*FDISP1*FDEXP((DAM*RATP1)**2)
      ZDISP2= CDEXP(2*ZI*Y*DAM*RATP2**2)*FDISP2*FDEXP((DAM*RATP2)**2)
C
C     FIRST MESON:
C     REAL MESON MASS:
      AM=PIM*DSQRT((AM1/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM1
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM1+RATM)
      EPM  =FDERFC( XLM1+RATM)
C
      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU1)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU1)*X1+(FIM-2*VKU1)*X2-(AM/PIM)*DHIM

      FIM1  = FIM
      DFIM1  = DFIM
      DDFIM1  = DDFIM

C     COMPLEX MESON MASS:
       IF(IYEX.EQ.1) THEN
      ZAM=PIM*CDSQRT((BM1/PIM)**2+Y*Y+2*ZI*DAM*Y)
      ZXM=ZAM*X/PIM
      ZRATM =ZAM/ALM1
      ZERATM=ZDEXP(ZRATM*ZRATM)
      ZEXM  =ZDEXP(ZXM)
      ZEMM  =ZDERFC(-XLM1+ZRATM)
      ZEPM  =ZDERFC( XLM1+ZRATM)

      ZFIM   = ZF0(ZERATM,ZEXM,ZEMM,ZEPM,X )
      ZHIM   = ZH0(ZERATM,ZEXM,ZEMM,ZEPM,X )
      ZDFIM  =-(ZFIM-2*VKU1)*X1-(ZAM/PIM)*ZHIM
      ZDHIM  =-(ZAM/PIM)*ZFIM-ZHIM*X1
      ZDDFIM =-(ZDFIM-2*DVKU1)*X1+(ZFIM-2*VKU1)*X2-(ZAM/PIM)*ZDHIM
       ENDIF
C     END FIRST MESON
C
C     SECOND MESON:
      IF(KIND.NE.4) THEN
C     REAL MESON MASS:
      AM=PIM*DSQRT((AM2/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM2
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM2+RATM)
      EPM  =FDERFC( XLM2+RATM)

      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU2)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU2)*X1+(FIM-2*VKU2)*X2-(AM/PIM)*DHIM

C     COMPLEX MESON MASS:
       IF(IYEX.EQ.2) THEN
      ZAM=PIM*CDSQRT((BM2/PIM)**2+Y*Y+2*ZI*DAM*Y)
      ZXM=ZAM*X/PIM
      ZRATM =ZAM/ALM2
      ZERATM=ZDEXP(ZRATM*ZRATM)
      ZEXM  =ZDEXP(ZXM)
      ZEMM  =ZDERFC(-XLM2+ZRATM)
      ZEPM  =ZDERFC( XLM2+ZRATM)

      ZFIM   = ZF0(ZERATM,ZEXM,ZEMM,ZEPM,X )
      ZHIM   = ZH0(ZERATM,ZEXM,ZEMM,ZEPM,X )
      ZDFIM  =-(ZFIM-2*VKU2)*X1-(ZAM/PIM)*ZHIM
      ZDHIM  =-(ZAM/PIM)*ZFIM-ZHIM*X1
      ZDDFIM =-(ZDFIM-2*DVKU2)*X1+(ZFIM-2*VKU2)*X2-(ZAM/PIM)*ZDHIM
       ENDIF
      ENDIF
      IF(KIND.EQ.4) THEN
      FIM = FI2
      DFIM = DFI2
      DDFIM = DDFI2
       IF(IYEX.EQ.2) THEN
      ZFIM = FI2
      ZDFIM = DFI2
      ZDDFIM = DDFI2
       ENDIF
      ENDIF
      FIM2  = FIM
      DFIM2  = DFIM
      DDFIM2  = DDFIM
C     END SECOND MESON

       IF(IYEX.EQ.2) THEN
C     FOR THE FIRST CONTRIBUTION:
      GEW1 = 0.5D0*GEW/Y**2
      GIM1 = (FI1-FDISP1*FIM1)
      DGIM1 = (DFI1-FDISP1*DFIM1)
      DDGIM1 = (DDFI1-FDISP1*DDFIM1)
      GIM2 = 2*DREAL(ZDISP2*ZFIM)
      DGIM2 = 2*DREAL(ZDISP2*ZDFIM)
      DDGIM2 = 2*DREAL(ZDISP2*ZDDFIM)
C     FOR THE SECOND CONTRIBUTION:
      GEW2 = ((Y*Y-DAM*DAM)/(Y*Y+DAM*DAM)**2)*GEW
      DFIM1 = DFI1
      DDFIM1 = DDFI1
      FIM2 = (FDISA*FI2-FDISP2*FIM2)
      DFIM2 = (FDISA*DFI2-FDISP2*DFIM2)
      DDFIM2 = (FDISA*DDFI2-FDISP2*DDFIM2)
       ENDIF
       IF(IYEX.EQ.1) THEN
C     FOR THE FIRST CONTRIBUTION:
      GEW1 = 0.5D0*GEW/Y**2
      GIM2 = (FI2-FDISP2*FIM2)
      DGIM2 = (DFI2-FDISP2*DFIM2)
      DDGIM2 = (DDFI2-FDISP2*DDFIM2)
      GIM1 = 2*DREAL(ZDISP1*ZFIM)
      DGIM1 = 2*DREAL(ZDISP1*ZDFIM)
      DDGIM1 = 2*DREAL(ZDISP1*ZDDFIM)
C     FOR THE SECOND CONTRIBUTION:
      GEW2 = ((Y*Y-DAM*DAM)/(Y*Y+DAM*DAM)**2)*GEW
      DFIM1 = (FDISA*DFI1-FDISP1*DFIM1)
      DDFIM1 = (FDISA*DDFI1-FDISP1*DDFIM1)
      FIM2 = FI2
      DFIM2 = DFI2
      DDFIM2 = DDFI2
       ENDIF
C
      IF(KIND.EQ.1) THEN
      FUN(1)  = FUN(1)  + GEW2*OPAEES(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAEES(GIM2,DGIM1,DDGIM1,X)
      FUN(11) = FUN(11) + GEW2*OCREES(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OCREES(GIM2,DGIM1,DDGIM1,X)
      FUN(2)  = FUN(2)  + GEW2*OPAEET(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAEET(GIM2,DGIM1,DDGIM1,X)
      FUN(12) = FUN(12) + GEW2*OCREET(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OCREET(GIM2,DGIM1,DDGIM1,X)
      FUN(3)  = FUN(3)  + GEW2*OPAEMT(DFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAEMT(DGIM2,DGIM1,DDGIM1,X)
      FUN(4)  = FUN(4)  + GEW2*OPAEMO(DFIM2,DFIM1,X)
     .                  + GEW1*OPAEMO(DGIM2,DGIM1,X)
      FUN(5)  = FUN(5)  + GEW2*OPAMMC(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAMMC(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(6)  = FUN(6)  + GEW2*OPAMMS(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAMMS(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(7)  = FUN(7)  + GEW2*OPAMMT(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPAMMT(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(8)  = FUN(8)  + GEW2*OPEMS2(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPEMS2(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(9)  = FUN(9)  + GEW2*OPEMT2(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPEMT2(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(10) = FUN(10) + GEW2*OPEMO2(DFIM2,DFIM1,X)
     .                  + GEW1*OPEMO2(DGIM2,DGIM1,X)
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1)  = FUN(1)  + GEW2*OSCS(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OSCS(GIM2,DGIM1,DDGIM1,X)
      FUN(2)  = FUN(2)  + GEW2*OSCT(FIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OSCT(GIM2,DGIM1,DDGIM1,X)
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1)  = FUN(1)  + GEW2*OPSC(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPSC(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(2)  = FUN(2)  + GEW2*OPSS(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPSS(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      FUN(3)  = FUN(3)  + GEW2*OPST(DFIM2,DDFIM2,DFIM1,DDFIM1,X)
     .                  + GEW1*OPST(DGIM2,DDGIM2,DGIM1,DDGIM1,X)
      ENDIF
C
110   CONTINUE
100   CONTINUE
C
C     CONTRIBUTIONS FROM Y>YMAX=YB(NINT+1)
C
      IF(IMETH.EQ.0) THEN
      YMAX = YB(NINT+1)
      YFAC2 = YMAX/(YMAX**2+DAM**2)*FDISA
      IF(KIND.EQ.1) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OPAEES(FI2,DFI1,DDFI1,X)*YFAC2
      FUN(11)= FUN(11)+(1.D0/PI)*OCREES(FI2,DFI1,DDFI1,X)*YFAC2
      FUN(2) = FUN(2)+(1.D0/PI)*OPAEET(FI2,DFI1,DDFI1,X)*YFAC2
      FUN(12)= FUN(2)+(1.D0/PI)*OCREET(FI2,DFI1,DDFI1,X)*YFAC2
      FUN(3) = FUN(3)+(1.D0/PI)*OPAEMT(DFI2,DFI1,DDFI1,X)*YFAC2
      FUN(4) = FUN(4)+(1.D0/PI)*OPAEMO(DFI2,DFI1,X)*YFAC2
      FUN(5) = FUN(5)+(1.D0/PI)*OPAMMC(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(6) = FUN(6)+(1.D0/PI)*OPAMMS(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(7) = FUN(7)+(1.D0/PI)*OPAMMT(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(8) = FUN(8)+(1.D0/PI)*OPEMS2(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(9) = FUN(9)+(1.D0/PI)*OPEMT2(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(10)= FUN(10)+(1.D0/PI)*OPEMO2(DFI2,DFI1,X)*YFAC2
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OSCS(FI2,DFI1,DDFI1,X)*YFAC2
      FUN(2) = FUN(2)+(1.D0/PI)*OSCT(FI2,DFI1,DDFI1,X)*YFAC2
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OPSC(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(2) = FUN(2)+(1.D0/PI)*OPSS(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      FUN(3) = FUN(3)+(1.D0/PI)*OPST(DFI2,DDFI2,DFI1,DDFI1,X)*YFAC2
      ENDIF
      ENDIF
CC
1000  CALL ERRSET(208,256,1,1)
      RETURN
      END

C **********************************************************************
      SUBROUTINE YNFUN3(ITYPE,KIND,X,AM1,AM2,ALM1,ALM2,DAMB,FUN)
C
C   THIS ROUTINE IS FOR THE CASE OF TWO(!) MESONS CARRYING STRANGENESS,
C   FUNCTIONS FOR KAON-KAON EXCHANGE ETC. , AM1=AM2= AMK= KAON MASS
C   VERSION: 23 FEBRUARI, 1996
C **********************************************************************
C
C     THIS ROUTINE IS CALLED FROM: YNPIBE.FORTRAN.C
C     IT IS UNDERSTOOD THAT AM1=PIM, SO INDEX 1 REFERS TO THE PION.
C     NOTICE: IN THIS ROUTINE ITYPE=1 (PLANAR) AND ITYPE=2(CROSSED)
C
C     COMPUTES THE FOURIER TRANSFORMS OF D_{X} AND D_{//}, WHERE
C
C     1) ITYPE=2: KAON-KAON EXCHANGE (CROSSED):
C
C                   1                1
C   -2* D_{X} = + --------  ---------------------- .
C                OM1.OM2  (OM1^2-A^2)(OM2^2-A^2)
C
C                 |                              1     |
C               X |(OM1+OM2) - (OM1.OM2+A^2) --------- |
C                 |                          OM1 + OM2 |
C
C     2) ITYPE=1: KAON-KAON EXCHANGE (PLANAR):
C
C                    2 A^2             |      1
C  2* D_{//}= ---------------------- . | --------------
C             (OM1^2-A^2)(OM2^2-A^2)   | OM2(OM2^2-A^2)
C
C                   1                   1        |
C           + -------------- + ------------------| - 2* D_{X}
C             OM1(OM1^2-A^2)   OM1.OM2(OM1 + OM2)|
C
C     FOR VARIOUS COMBINATIONS OF DERIVATIVES.
C
C     NOTICE: THE SAME AS YNFUN IN THE LIMIT DAM=0.D0 !
C
C     KIND = 1: PION-VECTOR-,   KIND = 2: PION-SCALAR-,
C     KIND = 3: PION-PSSCALAR-, KIND = 4: PION-POMERON-EXCHANGE.
C
C     SCHEMATICALLY:
C     PION-VECTOR EXCHANGE:
C     FUN(1) = OPAEES  FUN(2) = OPAEET  FUN(3) = OPAEMT1
C     FUN(4) = OPAEMO  FUN(5) = OPAMMC  FUN(6) = OPAMMS
C     FUN(7) = OPAMMT  FUN(8) = OPAEMS2 FUN(9) = OPAEMT2
C     FUN(10)= OPAEMO2 FUN(11)= OCREES  FUN(12)= OCREET 
C     PION-SCALAR EXCHANGE:
C     FUN(1) = OSCS    FUN(2) = OSCT
C     PION-PSSCALAR EXCHANGE:
C     FUN(1) = OPSC    FUN(2) = OPSS    FUN(3) = OPST
C  !! NOTE: simplest def. of functions here, no k^2/M^2-terms!
C           in OPAEES etc.
C
C     FOR FORM FACTOR TREATMENT SEE:
C     TH.A.RIJKEN, ANNALS OF PHYSICS 208, P.253 (1990).
C
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
***   COMMON/MASSES/PIM,AMNC(3),AMY(3),REDM(3),PLAB,AK(3),AKS(3),TEM(3)
      DIMENSION XX(20),WW(20),YB(3),FUN(12)
      DATA PI/3.14159265D0/,SRPI/1.7724538509D0/,ICALL/0/
      DATA YB/0.D0,10.D0,50.D0/,NINT/1/,NPNT/16/,IMETH/0/
      DATA PIM/138.041D0/,AMPRO/938.2796D0/
      SAVE XX,WW,DAM
C
C     ACCORDING TO THE PAPER APPENDIX B,
C     DEFINITION STATEMENT OPERATORS:
C
C 1)  PION-VECTOR EXCHANGE:
      OPAEES(FF,D1G,D2G,XH)=  2*FF*(2*D1G/XH+D2G)/3.D0
      OPAEET(FF,D1G,D2G,XH)= -2*FF*(D1G/XH-D2G)/3.D0
      OCREES(FF,D1G,D2G,XH)=
     .OPAEES(FF,D1G,D2G,XH)
      OCREET(FF,D1G,D2G,XH)=
     .OPAEET(FF,D1G,D2G,XH)

      OPAEMT(D1F,D1G,D2G,XH)=  2*D1F*(D1G/XH-D2G)/XH
      OPAEMO(D1F,D1G,XH)=  4*D1F*D1G/XH/XH
      OPEMS2(D1F,D2F,D1G,D2G,XH)=(2*D1F/XH+D2F)*(2*D1G/XH+D2G)/3.D0
      OPEMT2(D1F,D2F,D1G,D2G,XH)=(D1F/XH-D2F)*(D1G/XH-D2G)/3.D0
      OPEMO2(D1F,D1G,XH)= 2*D1F*D1G/XH/XH               

      OPAMMC(D1F,D2F,D1G,D2G,XH)=
     .2*( D1F*D1G/XH + D1F*D2G + D2F*D1G)/XH
      OPAMMS(D1F,D2F,D1G,D2G,XH)=
     .-2*( 3*D1F*D1G/XH/XH + D1F*D2G/XH + D2F*D1G/XH +D2F*D2G)/3.D0
      OPAMMT(D1F,D2F,D1G,D2G,XH)=
     .-(D1F/XH+D2F)*(D1G/XH-D2G)/3.D0+2*(D1F/XH-D2F)*D1G/XH/3.D0

C 2)  PION-SCALAR EXCHANGE:
      OSCS(FF,D1G,D2G,XH)= +FF*(2*D1G/XH+D2G)/3.D0
      OSCT(FF,D1G,D2G,XH)= -FF*(D1G/XH-D2G)/3.D0
      OSCO(D1F,D1G,XH)= 0.D0

C 3)  PION-PSEUDO-SCALAR EXCHANGE:
      OPSC(D1F,D2F,D1G,D2G,XH)= (2*D1F*D1G/XH/XH+D2F*D2G)
      OPSS(D1F,D2F,D1G,D2G,XH)= 2*(D1F*D1G/XH+D1F*D2G+D2F*D1G)/XH/3.D0
      OPST(D1F,D2F,D1G,D2G,XH)= 1*( (D1F/XH-D2F)*D1G/XH +
     . (D1G/XH-D2G)*D1F/XH )/3.D0
C
C     POTENTIAL FUNTION DEFINITION
C
      F0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH-EPH*EXH )/(2*XH)
      H0(ERATH,EXH,EMH,EPH,XH)=ERATH*( EMH/EXH+EPH*EXH )/(2*XH)
C
      CALL ERRSET(208,256,-1,1)
      IF(ICALL.EQ.0) THEN
        CALL GAUSS(NPNT,XX,WW,1)
        IF(IMETH.EQ.1) NINT=1
C       PRINT 52, IMETH,NPNT
C52     FORMAT(//,'  IN HFUN: IMETH=',I2,' NPNT=',I2,//)
        DAM = DAMB/PIM
        ICALL=1
      ENDIF
CC
      X1   = 1.D0/X
      X2   = X1*X1
CC
      RATP1=PIM/ALM1
      XLM1 =0.5D0*X*ALM1/PIM
      XLM12=XLM1*XLM1
      VKU1 =(ALM1/PIM)*FDEXP(-XLM12)/(2*SRPI)
      DVKU1=-0.5D0*(ALM1/PIM)**2*X*VKU1

      RATP2=PIM/ALM2
      XLM2 =0.5D0*X*ALM2/PIM
      XLM22=XLM2*XLM2
      VKU2 =(ALM2/PIM)*FDEXP(-XLM22)/(2*SRPI)
      DVKU2=-0.5D0*(ALM2/PIM)**2*X*VKU2

      BM1 = DSQRT(AM1**2-DAMB**2)
      XA   =X*BM1/PIM
      RATA =BM1/ALM1
      ERATA=DEXP(RATA*RATA)
      EXA  =DEXP(XA)
      EMA  =FDERFC(-XLM1 +RATA)
      EPA  =FDERFC( XLM1 +RATA)
      FIA  =F0(ERATA,EXA ,EMA ,EPA ,X)
      HIA  =H0(ERATA,EXA ,EMA ,EPA ,X)
      DFIA =-(FIA -2*VKU1)*X1-(BM1/PIM)*HIA
      DHIA  =-(BM1/PIM)*FIA -HIA *X1
      DDFIA  =-(DFIA-2*DVKU1)*X1+(FIA-2*VKU1)*X2-(BM1/PIM)*DHIA
      DDHIA =-(BM1/PIM)*DFIA-DHIA*X1+HIA*X2
C     D3FIA =-(DDFIA-2*DDVKUA)*X1+(DFIA-2*DVKUA)*X2-(BM1/PIM)*DDHIA
C    .       +(DFIA-2*DVKUA)*X2-2*(FIA-2*VKUA)*X3
C     D3HIA =-(BM1/PIM)*DDFIA-DDHIA*X1+2*DHIA*X2-2*HIA*X3
      F4A   = 0.5D0*(PIM/BM1)*X*HIA
      DF4A  = 0.5D0*(PIM/BM1)*(HIA+X*DHIA)
      DDF4A = 0.5D0*(PIM/BM1)*(2*DHIA+X*DDHIA)
C     D3F4A  = 0.5D0*(PIM/BM1)*(3*DDHIA+X*D3HIA)

      FI1= FIA
      DFI1= DFIA
      DDFI1= DDFIA
      FI41= F4A
      DFI41= DF4A
      DDFI41= DDF4A

      IF(KIND.NE.4) THEN
      BM2 = DSQRT(AM2**2-DAMB**2)
      XA   =X*BM2/PIM
      RATA =BM2/ALM2
      ERATA=DEXP(RATA*RATA)
      EXA  =DEXP(XA)
      EMA  =FDERFC(-XLM2 +RATA)
      EPA  =FDERFC( XLM2 +RATA)
      FIA  =F0(ERATA,EXA ,EMA ,EPA ,X)
      HIA  =H0(ERATA,EXA ,EMA ,EPA ,X)
      DFIA =-(FIA -2*VKU2)*X1-(BM2/PIM)*HIA
      DHIA  =-(BM2/PIM)*FIA -HIA *X1
      DDFIA  =-(DFIA-2*DVKU2)*X1+(FIA-2*VKU2)*X2-(BM2/PIM)*DHIA
      DDHIA =-(BM2/PIM)*DFIA-DHIA*X1+HIA*X2

      F4A   = 0.5D0*(PIM/BM2)*X*HIA
      DF4A  = 0.5D0*(PIM/BM2)*(HIA+X*DHIA)
      DDF4A = 0.5D0*(PIM/BM2)*(2*DHIA+X*DDHIA)
      ENDIF
      IF(KIND.EQ.4) THEN
      XA   =X*AM2/PIM
      FIA  =(4.D0/SRPI)*(AM2/PIM)*(AM2/AMPRO)**2*FDEXP(-XA*XA)
      DFIA  = -2*(AM2/PIM)*XA*FIA
      DDFIA = -2*(AM2/PIM)*(XA*DFIA + DFIA*AM2/PIM)
C     D3FIA = -2*(AM2/PIM)*(XA*DDFIA + 2*DFIA*AM2/PIM)
      RATD  = 0.5D0*PIM/AM2
      F4A  = FIA*RATD**2
      DF4A = DFIA*RATD**2
      DDF4A= DDFIA*RATD**2
C     D3F4A= D3FIA*RATD**2
      ENDIF
      FI2= FIA
      DFI2= DFIA
      DDFI2= DDFIA
      FI42= F4A
      DFI42= DF4A
      DDFI42= DDF4A
C
C     CONSTRUCTION BASE FUNCTIONS BY INTEGRATION
C
      DO 105 IFUN=1,12
  105 FUN(IFUN) = 0.D0
      FDISA=FDEXP(+DAM**2*RATP1**2)*FDEXP(+DAM**2*RATP2**2)
      IF(ITYPE.EQ.1) THEN
      IF(KIND.EQ.1) THEN
      FUN(1)  =  1*DAM*(OPAEES(FI42,DFI1,DDFI1,X)
     .               +OPAEES(FI2,DFI41,DDFI41,X))*FDISA
      FUN(11) =  1*DAM*(OCREES(FI42,DFI1,DDFI1,X)
     .               +OCREES(FI2,DFI41,DDFI41,X))*FDISA
      FUN(2)  =  1*DAM*(OPAEET(FI42,DFI1,DDFI1,X)
     .               +OPAEET(FI2,DFI41,DDFI41,X))*FDISA
      FUN(12) =  1*DAM*(OCREET(FI42,DFI1,DDFI1,X)
     .               +OCREET(FI2,DFI41,DDFI41,X))*FDISA
      FUN(3)  =  1*DAM*(OPAEMT(DFI42,DFI1,DDFI1,X)
     .               +OPAEMT(DFI2,DFI41,DDFI41,X))*FDISA
      FUN(4)  =  1*DAM*(OPAEMO(DFI42,DFI1,X)
     .               +OPAEMO(DFI2,DFI41,X))*FDISA
      FUN(5)  =  1*DAM*(OPAMMC(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPAMMC(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(6)  =  1*DAM*(OPAMMS(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPAMMS(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(7)  =  1*DAM*(OPAMMT(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPAMMT(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(8)  =  1*DAM*(OPEMS2(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPEMS2(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(9)  =  1*DAM*(OPEMT2(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPEMT2(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(10) =  1*DAM*(OPEMO2(DFI42,DFI1,X)
     .               +OPEMO2(DFI2,DFI41,X))*FDISA
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1)  =  1*DAM*(OSCS(FI42,DFI1,DDFI1,X)
     .               +OSCS(FI2,DFI41,DDFI41,X))*FDISA
      FUN(2)  =  1*DAM*(OSCT(FI42,DFI1,DDFI1,X)
     .               +OSCT(FI2,DFI41,DDFI41,X))*FDISA
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1)  =  1*DAM*(OPSC(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPSC(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(2)  =  1*DAM*(OPSS(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPSS(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      FUN(3)  =  1*DAM*(OPST(DFI42,DDFI42,DFI1,DDFI1,X)
     .               +OPST(DFI2,DDFI2,DFI41,DDFI41,X))*FDISA
      ENDIF
      ENDIF
      DO 100 INT=1,NINT
      ABM=0.5D0*(YB(INT+1)-YB(INT))
      ABP=0.5D0*(YB(INT+1)+YB(INT))
      DO 110 IY=1,NPNT
      IF(IMETH.EQ.0) THEN
      Y=ABM*XX(IY)+ABP
      GEW=WW(IY)*ABM*1.D0/PI
      ENDIF
      IF(IMETH.EQ.1) THEN
      Y=(1.D0+XX(IY))/(1.D0-XX(IY))
      GEW=WW(IY)*(2.D0/(1.D0-XX(IY))**2)*1.D0/PI
      ENDIF
      IF(Y.GT.50.D0) GOTO 110 
C     FACTOR FROM FORM FACTOR EFFECTS THROUGH DISP. REL. :
      FDISP1=FDEXP(-Y*Y*RATP1*RATP1)
      FDISP2=FDEXP(-Y*Y*RATP2*RATP2)
      FDISP = FDISP1*FDISP2
C
      AM=PIM*DSQRT((AM1/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM1
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM1+RATM)
      EPM  =FDERFC( XLM1+RATM)
C
      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU1)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU1)*X1+(FIM-2*VKU1)*X2-(AM/PIM)*DHIM

      FIM1  = FIM
      DFIM1  = DFIM
      DDFIM1  = DDFIM
C
      IF(KIND.NE.4) THEN
      AM=PIM*DSQRT((AM2/PIM)**2+Y*Y)
      XM=AM*X/PIM
      RATM =AM/ALM2
      ERATM=FDEXP(RATM*RATM)
      EXM  =FDEXP(XM)
      EMM  =FDERFC(-XLM2+RATM)
      EPM  =FDERFC( XLM2+RATM)

      FIM   = F0(ERATM,EXM,EMM,EPM,X )
      HIM   = H0(ERATM,EXM,EMM,EPM,X )
      DFIM  =-(FIM-2*VKU2)*X1-(AM/PIM)*HIM
      DHIM  =-(AM/PIM)*FIM-HIM*X1
      DDFIM =-(DFIM-2*DVKU2)*X1+(FIM-2*VKU2)*X2-(AM/PIM)*DHIM

      ENDIF
      IF(KIND.EQ.4) THEN
      FIM = FI2
      DFIM = DFI2
      DDFIM = DDFI2
      ENDIF
      FIM2  = FIM
      DFIM2  = DFIM
      DDFIM2  = DDFIM

      IF(ITYPE.EQ.1) GEWH = GEW*(Y**2+3*DAM**2)/(Y**2+DAM**2)**2
      IF(ITYPE.EQ.2) GEWH = +GEW/(Y**2+DAM**2)
C
      IF(KIND.EQ.1) THEN
      FUN(1)  = FUN(1)  + GEWH*(FDISA*OPAEES(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAEES(FIM2,DFIM1,DDFIM1,X))
      FUN(11) = FUN(11) + GEWH*(FDISA*OCREES(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OCREES(FIM2,DFIM1,DDFIM1,X))
      FUN(2)  = FUN(2)  + GEWH*(FDISA*OPAEET(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAEET(FIM2,DFIM1,DDFIM1,X))
      FUN(12) = FUN(12) + GEWH*(FDISA*OCREET(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OCREET(FIM2,DFIM1,DDFIM1,X))
      FUN(3)  = FUN(3)  + GEWH*(FDISA*OPAEMT(DFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAEMT(DFIM2,DFIM1,DDFIM1,X))
      FUN(4)  = FUN(4)  + GEWH*(FDISA*OPAEMO(DFI2,DFI1,X)
     .                  -FDISP*OPAEMO(DFIM2,DFIM1,X))
      FUN(5)  = FUN(5)  + GEWH*(FDISA*OPAMMC(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAMMC(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(6)  = FUN(6)  + GEWH*(FDISA*OPAMMS(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAMMS(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(7)  = FUN(7)  + GEWH*(FDISA*OPAMMT(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPAMMT(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(8)  = FUN(8)  + GEWH*(FDISA*OPEMS2(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPEMS2(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(9)  = FUN(9)  + GEWH*(FDISA*OPEMT2(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPEMT2(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(10) = FUN(10) + GEWH*(FDISA*OPEMO2(DFI2,DFI1,X)
     .                  -FDISP*OPEMO2(DFIM2,DFIM1,X))
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1)  = FUN(1)  + GEWH*(FDISA*OSCS(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OSCS(FIM2,DFIM1,DDFIM1,X))
      FUN(2)  = FUN(2)  + GEWH*(FDISA*OSCT(FI2,DFI1,DDFI1,X)
     .                  -FDISP*OSCT(FIM2,DFIM1,DDFIM1,X))
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1)  = FUN(1)  + GEWH*(FDISA*OPSC(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPSC(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(2)  = FUN(2)  + GEWH*(FDISA*OPSS(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPSS(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      FUN(3)  = FUN(3)  + GEWH*(FDISA*OPST(DFI2,DDFI2,DFI1,DDFI1,X)
     .                  -FDISP*OPST(DFIM2,DDFIM2,DFIM1,DDFIM1,X))
      ENDIF
C
110   CONTINUE
100   CONTINUE
C
C     CONTRIBUTIONS FROM Y>YMAX=YB(NINT+1)
C
      IF(IMETH.EQ.0) THEN
      YMAX = YB(NINT+1)
      IF(DAM.EQ.0.D0) YFACH = 1.D0/YMAX
      IF(DAM.NE.0.D0) THEN
      IF(ITYPE.EQ.1) YFACH=+(PI-2*DATAN(YMAX/DAM))/DAM
     .                     -YMAX/(YMAX**2+DAM**2)
      IF(ITYPE.EQ.2) YFACH=+(PI/2.D0-DATAN(YMAX/DAM))/DAM
      ENDIF
      YFACH = FDISA*YFACH
      IF(KIND.EQ.1) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OPAEES(FI2,DFI1,DDFI1,X)*YFACH
      FUN(11)= FUN(11)+(1.D0/PI)*OCREES(FI2,DFI1,DDFI1,X)*YFACH
      FUN(2) = FUN(2)+(1.D0/PI)*OPAEET(FI2,DFI1,DDFI1,X)*YFACH
      FUN(12)= FUN(12)+(1.D0/PI)*OCREET(FI2,DFI1,DDFI1,X)*YFACH
      FUN(3) = FUN(3)+(1.D0/PI)*OPAEMT(DFI2,DFI1,DDFI1,X)*YFACH
      FUN(4) = FUN(4)+(1.D0/PI)*OPAEMO(DFI2,DFI1,X)*YFACH
      FUN(5) = FUN(5)+(1.D0/PI)*OPAMMC(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(6) = FUN(6)+(1.D0/PI)*OPAMMS(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(7) = FUN(7)+(1.D0/PI)*OPAMMT(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(8) = FUN(8)+(1.D0/PI)*OPEMS2(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(9) = FUN(9)+(1.D0/PI)*OPEMT2(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(10)= FUN(10)+(1.D0/PI)*OPEMO2(DFI2,DFI1,X)*YFACH
      ENDIF
C
      IF(KIND.EQ.2.OR.KIND.EQ.4) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OSCS(FI2,DFI1,DDFI1,X)*YFACH
      FUN(2) = FUN(2)+(1.D0/PI)*OSCT(FI2,DFI1,DDFI1,X)*YFACH
      ENDIF
C
      IF(KIND.EQ.3) THEN
      FUN(1) = FUN(1)+(1.D0/PI)*OPSC(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(2) = FUN(2)+(1.D0/PI)*OPSS(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      FUN(3) = FUN(3)+(1.D0/PI)*OPST(DFI2,DDFI2,DFI1,DDFI1,X)*YFACH
      ENDIF
      ENDIF
CC
1000  CALL ERRSET(208,256,1,1)
      RETURN
      END

C**************************************************************
      FUNCTION ZDERFC(ZP)
C**************************************************************

      IMPLICIT REAL*8 (A-H,O-Y)
      COMPLEX*16 ZP,Z,IMA,ZDERFC,ZDEXP,Z2,WIZ
      INTEGER CAPN,NU,N,NP1
      REAL*8 H,H2,LAMBDA,R1,R2,S,S1,S2,T1,T2,C,REZ,IMZ,X,Y
      LOGICAL B
      DATA IMA/(0D0,1D0)/

      X =DREAL(IMA*ZP)
      Y =DIMAG(IMA*ZP)

C  FIRST QUADRANT:
      IF(X .GE.0.D0 .AND. Y .GE.0.D0) THEN
      ICASE = 1
      ENDIF
C  SECOND QUADRANT:
      IF(X .LT.0.D0 .AND. Y .GE.0.D0) THEN
      ICASE = 2
      X = -X
      ENDIF
C  THIRD QUADRANT:
      IF(X .LT.0.D0 .AND. Y .LT.0.D0) THEN
      ICASE = 3
      X = -X
      Y = -Y
      ENDIF
C  FOURTH QUADRANT:
      IF(X .GE.0.D0 .AND. Y .LT.0.D0) THEN
      ICASE = 4
      Y = -Y
      ENDIF

      Z = X + IMA*Y

C  SO: Z= X + IY ALWAYS IN FIRST QUADRANT -> APPLICATION
C  GAUTSCHI ALGORITHM
**
**    Ref. W. Gautschi, Algorithm 363, Communications of the
**    ACM Vol. 12 (1969) 635.
**
      B =.FALSE.
      COEF = 1.12837916709551D0

C     IF (X.LE.-4D0) GOTO 2

      IF ((Y.LT.4.29D0).AND.(X.LT.5.33D0)) THEN
        S = (1D0-Y/4.29D0)*DSQRT(1D0-X*X/28.41D0)
        H = 1.6 * S
        H2 = 2*H
        CAPN = 6 + 23*S
        LAMBDA = H2**CAPN
        NU = 9 + 21*S
      ELSE
        H = 0
        CAPN = 0
        NU = 8
      ENDIF
** lambda = 0 covers the underflow case when h > 0 is very small
      B = (H.EQ.0.OR.LAMBDA.EQ.0)
      R1 = 0D0
      R2 = 0D0
      S1 = 0D0
      S2 = 0D0
      DO 100 N =NU,0,-1
        NP1 = N + 1
        T1 = Y + H +NP1*R1
        T2 = X - NP1*R2
        C = 0.5D0/(T1*T1+T2*T2)
        R1=C*T1
        R2=C*T2
        IF (H.GT.0.AND.N.LE.CAPN) THEN
          T1 = LAMBDA+S1
          S1 = R1*T1-R2*S2
          S2=R2*T1+R1*S2
          LAMBDA=LAMBDA/H2
        ENDIF
 100  CONTINUE
      IF (Y.EQ.0) THEN
        XX = -X*X
        REZ = FDEXP(XX)
      ELSE
        IF (B) REZ = COEF *R1
        IF (.NOT.B) REZ = COEF*S1
      ENDIF
      IF (B) IMZ = COEF*R2
      IF (.NOT.B) IMZ = COEF*S2

      IF(ICASE.EQ.1) THEN
      WIZ  = REZ + IMA*IMZ
      ZDERFC = ZDEXP(-ZP*ZP)*WIZ
      ENDIF
      IF(ICASE.EQ.2) THEN
      WIZ  = REZ - IMA*IMZ
      ZDERFC = ZDEXP(-ZP*ZP)*WIZ
      ENDIF
      IF(ICASE.EQ.3) THEN
      ZDERFC = 2.D0 - (REZ + IMA*IMZ) * ZDEXP(-ZP*ZP)
      ENDIF
      IF(ICASE.EQ.4) THEN
      ZDERFC = 2.D0 - (REZ - IMA*IMZ) * ZDEXP(-ZP*ZP)
      ENDIF

      RETURN

C 1   ZDERFC=DCMPLX(0D0)
C     RETURN
C 2   ZDERFC=DCMPLX(2D0)
C     RETURN
      END

C**************************************************************
      FUNCTION ZDEXP(Z)
C**************************************************************
      IMPLICIT REAL*8(A-Y)
      COMPLEX*16  ZDEXP,IMA,Z
      IMA=(0D0,1D0)
      X =DREAL(Z)
      Y =DIMAG(Z)
      ZDEXP = FDEXP(X)*(DCOS(Y)+IMA*DSIN(Y))
      RETURN
      END


