C *******************************************************************
C     SUBROUTINE HNPOT04(IMODEL,X,LINT,JSPIN,NCHAN,NBASE,IGREEN,NS)
C *******************************************************************
C *******************************************************************
c     version: nchan=1 -> Sigma(+)-proton
c              nchan=2 -> Lambda  -neutron
c              nchan=3 -> Lambda  -proton  
c     note: sigma(-)-proton, not proper threshold treatment!!!
c--------------------------------------------------------------------
c     version with green transformation facility!!

c     IGREEN = 0: output the model-potentials, unmodified !                   
c   
c     IGREEN = 1: green transformation -> modified potentials  

C *******************************************************************
C
C     HYPERON - NUCLEON POTENTIAL PROGRAM :
C
C     Model-options:
C     *********************************************************************
C     *  ESC04: OSAKA-VERSION DECEMBER 2004                               *
C     *                                                                   *
C     *  IMODEL = 1 : Hard-Core MODEL PRD15 = YN-model D                  *
C     *  IMODEL = 2 : Hard-Core MODEL PRD20 = YN-model F                  *
C     *                                                                   *
C     *  IMODEL = 3 : Soft-Core MODEL NSC97f= YN-model                    *
C     *  IMODEL = 4 : Soft-Core MODEL NSC97e= YN-model                    *
C     *                                                                   *
C     *  IMODEL = 6 : Soft-Core MODEL ESC04A= NN+YN-model, paper II       *
C     *  IMODEL = 7 : Soft-Core MODEL ESC04B= NN+YN-model, paper II       *
C     *  IMODEL = 8 : Soft-Core MODEL ESC04C= NN+YN-model, paper II       *
C     *  IMODEL = 9 : Soft-Core MODEL ESC04D= NN+YN-model, paper II       *
C                                                                         *
C     *********************************************************************
C
C     IN THIS VERSION ZERO'S IN FORM FACTORS ARE OPTIONAL !
C
C -------------------------------------------------------------------------
CC
CC    THE POTENTIALS ARE TO BE USED IN THE MULTICHANNEL SCHRODINGER
CC                                _    1
CC  [ 1/(2 mu )*d(i,j)*LAPL - V  (r) + -(LAPL*phi  (r) + phi  (r)*LAPL)
CC           i                 ij      2         ij         ij
CC       2                        _
CC   + (k /2*mu )*d(i,j) ] * psi (r) = 0 .
CC             i                j
CC
CC    here phi = FI in program.   LAPL = laplacian operator.
CC
CC    EQUATION LIKE NR.36 IN PR.D17,1978,
CC !! except for definition phi !! The adaption is made
C *******************************************************************
C
C  !! POTENTIALS FROM INDIVIDUAL MESONS CAN BE OBTAINED BY
C     CHANGING DATA OF COMMON/INMES/ IN BLOCKDATA SUBPROGRAM !!
C
C INPUT :
C     X     = YN-DISTANCE IN PION-COMPTON WAVELENGTH
C     NCHAN = 1 : SIGMA(+)-PROTON
C           = 2 : LAMBDA-NEUTRON
C           = 3 : LAMBDA-PROTON
C
C     NBASE = 1 : POTENTIALS ONLY CALCULATED ON THE ISOSPIN-BASIS
C           = 2 : POTENTIALS ALSO CALCULATED ON THE PARTICLE-BASIS
C
C     NS    = 0 : NOPRINT
C           = 1 : RESTRICTED PRINTING
C           = 2 : EXTENSIVE PRINTING. THE PROGRAM PRINTS X,
C                 AND THE TOTAL POTENTIALS ON THE ISOSPIN-BASIS
C               [  VCLL,VSIGLL,VTENLL,VSOLL,VSO2LL,FILL,DFILL,DDFILL ]
C     ISPIN=1/2 [  VCSS,VSIGSS,VTENSS,VSOSS,VSO2SS,FISS,DFISS,DDFISS ]
C               [  VCLS,VSIGLS,VTENLS,VSOLS,VSO2LS,FILS,DFILS,DDFILS ]
C
C     ISPIN=3/2 [  VCDR,VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR ]
C
C                 AND FOR NBASE=2 ALSO THE POTENTIALS ON THE
C                 PARTICLE BASIS
C
C     NS    = 3 : EXTRA PRINTING : THE PROGRAM PRINTS ALSO
C                 THE POTENTIALS ON THE ISOSPIN-BASIS PER MESON-TYPE
C
C     LINT = ORBITAL ANGULAR MOMENTUM YN-CHANNEL
C
C     JSPIN= TOTAL SPIN YN-CHANNEL
C
C     HERE: AKS(I) ARE PUT TO ZERO, RELEVANT IN CASE OF NADIA=1
C
C
C END INPUT DESCRIPTION. ********************************************
C
C OUTPUT: NIJMEGEN-SOFTCORE HYPERON-NUCLEON POTENTIALS
C
C     LITERATURE NIJMEGEN OBE-MODEL :  PHYS.REV. D17,P.768,1978 (NN)
C                                   :  FONTEFRAUD CONF.   ,1987 (YN)
C                                   :  PHYS.REV.,TO BE SUBMITTED(YN)
C
C AD NBASE=1 :
C     THE POTENTIALS ON THE ISOSPIN-BASIS ARE COMMUNICATED TO THE
C     CALLING PROGRAM VIA
C     COMMON/POT/VCLL,VSIGLL,VTENLL,VSOLL,VSO2LL,FILL,DFILL,DDFILL,
C    .           VCSS,VSIGSS,VTENSS,VSOSS,VSO2SS,FISS,DFISS,DDFISS,
C    .           VCLS,VSIGLS,VTENLS,VSOLS,VSO2LS,FILS,DFILS,DDFILS,
C    .           VCDR,VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR.
C    .           FISLL,DFISLL,DDFSLL,FISSS,DFISSS,DDFSSS,
C    .           FISLS,DFISLS,DDFSLS,FISDR,DFISDR,DDFSDR,
C    .           VASOLL,VASOSS,VASOLS,VASODR.
C
C     HERE NOMENCLATURE :
C
C     VC   = CENTRAL POTENTIAL   , VSIG = SPIN-SPIN POTENTIAL ,
C     VTEN = TENSOR     ,,       , VSO  = SPIN-ORBIT   ,,     ,
C     VSO2 = QUADRATIC SPIN-ORBIT  POTENTIAL.
C     FI   = PHI
C     DFI  = PHI'.
C     DDFI = PHI''.
C
C     PHI , PHI', AND PHI'' APPEAR IN THE SCHRODINGER EQUATION DUE
C     TO THE Q**2-DEPENDENCE OF THE POTENTIALS (SEE PR.D17,EQ.36),
C     NOTE HOWEVER THAT WE ADAPTED THE DEFINITION OF FI TO THE
C     MULTI-CHANNEL CASE (SEE BELOW FOR THE DESCRIPTION) !!!!
C
C     NOTE THAT WE DID INCLUDE THE ASO-POTENTIALS. THEY ARE,
C     IN THE PRESENT MODEL, UNIMPORTANT TO FIT THE LOW ENERGY YN-DATA,
C     HOWEVER THEY ARE IMPORTANT IN HYPERNUCLEI AND NUCLEAR MATTER.     
C
C     LL = LAMBDA-LAMBDA MATRIX ELEMENT  )
C     LS = LAMBDA-SIGMA    ,,     ,,     )  I=1/2
C     SS = SIGMA -SIGMA    ,,     ,,     )
C     DR = SIGMA -SIGMA    ,,     ,,        I=3/2
C
C AD NBASE=2 :
C     THE POTENTIALS ON THE PARTICLE-BASIS ARE COMMUNICATED TO THE
C     CALLING PROGRAM
C  I) FOR NCHAN=2 AND 3 VIA :
C     COMMON/POTPRT/VPRT(8,3,3)        PRT=PARTICLE
C
C     HERE NOMENCLATURE :
C     VPRT(1,*,*)= CENTRAL POTENTIAL , VPRT(2,*,*)= SPIN-SPIN POTENTIAL
C     VPRT(3,*,*)= TENSOR     ,,     , VPRT(4,*,*)= SPIN-ORBIT   ,,
C     VPRT(5,*,*)= QUADRATIC SPIN-ORBIT  POTENTIAL.
C     VPRT(6,*,*)= PHI               , VPRT(7,*,*)= PHI' ,
C     VPRT(8,*,*)= PHI''.
C
C     NCHAN=2  LABELS CHANNELS :
C     LAMBDA-NEUTRON=1, SIGMA(0)-NEUTRON=2, SIGMA(-)-PROTON=3
C     NCHAN=3  LABELS CHANNELS :
C     LAMBDA-PROTON =1, SIGMA(+)-NEUTRON=2, SIGMA(0)-PROTON=3
C
C II) FOR NCHAN=1 (SIGMA(+)-PROTON CHANNEL) VIA
C     COMMON/POT/, NAMELY THROUGH THE POTENTIALS
C     VCDR,VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR .
C
C
C
C     NOTE THAT IN THE NSC/ESC-MODEL
C          1) FOR YN WE DO THE GREEN TRANSF. ON THE ISOSPIN BASIS
C          2) FOR YN AND ISPIN=1/2 THE GREEN TRANSFORMATION HAS
C             TO BE DONE NUMERICALLY (2-CHANNEL PROBLEM)
C
C
C END OUTPUT DESCRIPTION. *******************************************
C
C     UNITS :  X IN PIM**-1, USED PIM=138.041 MEV
C              VC,VSIG,VTEN,VSO,VSO2 IN MEV**+1
C              FI IN MEV**-1 , DFI IN MEV**0 , DDFI IN MEV**+1
C
C ******************************************************************
C  Parameters: Organization DEC.2004
C ******************************************************************
C
C   PAR( 1,I)= XS     G1     FD1    FV1   GS1    GA2    FA1    GSPAIR
C   PAR( 2,I)= XT     ALP    ALVD   ALVV  GS*    PSID   ALPA   GVPAIR
C   PAR( 3,I)= X1P    GETAP  GOM    FOM   GEPS   GPOM   FE1    FVPAIR
C   PAR( 4,I)= X3P27  PIONC  AM1RO  ARO   AMD    1.0    THA    GAPAIR
C   PAR( 5,I)= XHIGL  PION0  AM2RO  BRO   AM1SI  ASI    GT1    HAPAIR
C   PAR( 6,I)= PROTON ETA    AMOM   ELPHI AM2SI  BSI    GEFF   GPPAIR
C   PAR( 7,I)= NEUTRN AMX0   AMFI   PSIS1 AM1ST  1.0    FT1    HOPAIR
C   PAR( 8,I)= ALM27  ALMP1  ALMV1  PSIS8 ALMS1  AMA2   FEFF   GEPAIR
C   PAR( 9,I)= ALM8S  ALMP0  ALMV0  THS1  ALMS0  AMPOM  ALMAX  GPIET     
C   PAR(10,I)= ALM10  THPS   THV    THS2  AMEPS  THD    SONV   GPIETP
C   PAR(11,I)= ALM8A  THPE   THRO   THDE  GAMEP  AHYPC  SONPV  ALPAPR
C   PAR(12,I)= ALM10S APV    FDNP1  FDNN1 GEPS2  UPOM   SONSC  ALSCPR
C   PAR(13,I)= ALM1   HFPI   HFD1   HFV1  ALS2   ---    SONVPR ALVDPR
C   PAR(14,I)= ---    FCHPI  HALVD  HALVV ---    ---    SONPPR ALVVPR 
C   PAR(15,I)= RSCAL  HFETP  HGOM   HFOM  ALS    GPOPP  SONSPR GPIPOM

C   PAR(16,I)= ---    ALMKA  ALMKS  ALMKP AMSCK  AMKSS  FB1    GTENPR
C   PAR(17,I)= ---    ---    ---    ---   ---    ---    ALFBX  FTENPR
C   PAR(18,I)= ---    ---    ---    ---   GPPS2  ---    FB9    GPSPR
C   PAR(19,I)= ---    ---    ---    ---   ---    ---    SONVA  GPIROV
C   PAR(20,I)= ---    ---    ---    ---   ---    ---    ---    GPIOMV

C     HERE : PHYSICAL COUPLINGS, EXCEPT FOR
C            THE SU3-SINGLET COUPLINGS GPS1 AND FD1
C
C ******************************************************************
C
C     ALM27 = ALAM IRREP 27      , ALAM10 = ALM IRREP 10
C     ALM8S = ALAM IRREP 27+8S   , ALM8A = ALAM IRREP 10*+8A
C
C     PAR(7,1)= ALAM IS CURRENT FORM FACTOR MASS SET IN OBEPOT
C               EQUAL TO ALAM** OF THE PARTICULAR CHANNEL
C
C     PARAMETERS FOR MODEL D AND F :
C     PAR(1-3,1)=XS,XT,XP : HARDCORE PARAMETERS ( 0 IN NSC-MODEL )
C     PAR(4-6,1)=X3P27,X3P8,XHIGHL: HARDCORE PARAMETERS ( 0 IN NSC-MODEL
C
C     G1,G6,G9 : PION-,ETA-,ETA'-COUPLING CONSTANT
C     FD1,ALVD,FD9 : VECTOR MESON G-TYPE COUPLING PARAMETERS
C     FV1,FV6,FV9 : VECTOR MESON F-TYPE COUPLING CONSTANTS
C     GS1,GS6,GS9 : SCALAR MESON COUPLING CONSTANTS
C     GPOM,GA2 : (POMERON+F+F')-,A2-COUPLING CONSTANT
C     THPS,THV,THS : PSEUDOSCALAR-,VECTOR-,SCALAR-MESON MIXING ANGLES
C     ARO,AM1RO,BRO,AM2RO : BROAD RHO-MESON PARAMETERS (SEE P.R.D17)
C     ASI,AM1SI,BSI,AM2SI :  ,,   EPS- ,,      ,,,            ,,
C     AMD,AMSST : DELTA-,S(975)-MESON MASS
C     ALAM : FORMFACTOR MASS
C
C     ALMNN ALMLL ALMSS ALMLS ALMLK  AMKSS   ALMSK : F.F. PER VERTEX
C     IN NSC SET IN OBEPOT TO APPROPRIATE VALUES.
C
C     PIM = PION MASS
C
c *******************************************************************
C BEGIN MAIN PROGRAM ************************************************
C *******************************************************************
      SUBROUTINE HNPOT04(IMODEL,X,LINT,JSPIN,NCHAN,NBASE,IGREEN,NS)
C *******************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 MPROT,MNEUT,MLAMB,MSIGP,MSIGN,MSIGM
      COMMON/MODEL/IMODL,INRS,ICNSTR,IRET,IDAM,IGERST,IGRTST
      COMMON/ONEBOS/IPSC,IVCT,ISCL,IDIF,IAXI,ITEN
      COMMON/TWOMES/IPAIR,ITPS,IPIBE,IPSBE
      COMMON/ZEROS/IZPS,IZVC,IZSC,IZAX,IZTEN
      COMMON/PAROOM/APV,IPV,IGM,INA,IOFF,NADIA    
      COMMON/PRMTRS/ PAR(20,8)
      COMMON/MASSES/PIM,AM(3),AMY(3),REDM(3),PLAB,AK(3),AKS(3),TEM(3)
      COMMON/DYNMAS/REDMM,AMLN,AMSN,AMH,AMSS,AMLS,AMNN
      COMMON/FRMFAC/ALM(7)
      COMMON/FORMF/ALAM,ALMP8,ALMV8,ALMS8,ALMP1,ALMV1,ALMS1,
     .             ALAMK,ALMKS,ALMKAP
      COMMON/ALLPS/AME,AMX,AMK,BMK
      COMMON/ALLVC/ARO,BRO,AM1RO,AM2RO,AMOM,AMFI,AMKS,BMKS
      COMMON/ALLSC/ASI,BSI,AM1SI,AM2SI,ASST,BSST,AMSST,AM2ST,AMD,AMSCK
      COMMON/ALLDF/AMPOM,AMF2,AMA2,AMKSS,GDA(11)
      COMMON/ALLAX/ FA(11),FB(11),ALMAX                            
      COMMON/COUPL/F1,F2,F3,F4,F5,F6,F7,F8,F9,F10,F11,
     .  G1,G2,G3,G4,G5,G6,G7,G8,G9,G10,G11,
     .  FD1,FD2,FD3,FD4,FD5,FD6,FD7,FD8,FD9,FD10,FD11,
     .  FV1,FV2,FV3,FV4,FV5,FV6,FV7,FV8,FV9,FV10,FV11,
     .  GS1,GS2,GS3,GS4,GS5,GS6,GS7,GS8,GS9,GS10,GS11,
     .  GD1,GD2,GD3,GD4,GD5,GD6,GD7,GD8,GD9,GD10,GD11,P
      COMMON/POT/VCLL,VSIGLL,VTENLL,VSOLL,VSO2LL,FILL,DFILL,DDFILL,
     .           VCSS,VSIGSS,VTENSS,VSOSS,VSO2SS,FISS,DFISS,DDFISS,
     .           VCLS,VSIGLS,VTENLS,VSOLS,VSO2LS,FILS,DFILS,DDFILS,
     .           VCDR,VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR,
     .           FISLL,DFISLL,DDFSLL,FISSS,DFISSS,DDFSSS,
     .           FISLS,DFISLS,DDFSLS,FISDR,DFISDR,DDFSDR,
     .           VASOLL,VASOSS,VASOLS,VASODR
*    .          ,VCNN,VSIGNN,VTENNN,VSONN,VSO2NN
      COMMON/PIMAS/PIM0
      DIMENSION VPRT(8,3,3)
      DIMENSION VI(8,4),BM(3,3),BMP(3,3)
      EQUIVALENCE (VI(1,1),VCLL)
      CHARACTER AMODEL(9)*9,SYS(3)*16,TYPV(8)*16              
      DATA SYS/'SIGMA(+)-PROTON','LAMBDA-NEUTRON','LAMBDA-PROTON'/
      DATA TYPV/'CENTRAL','SPIN-SPIN','TENSOR','SPIN-ORBIT',
     .          'QUADR SPIN-ORBIT','PHI-FUNCTION','DPHI-FUNCTION',
     .          'DDPHI-FUNCTION'/
      DATA N2/6/,ICALL/0/,NCHNH/-10/,LINTH/-10/,JSPINH/-10/,BM/9*0.D0/
      DATA MPROT/938.2796D0/,MNEUT/939.5731D0/,MLAMB/1115.60D0/,
     .     MSIGP/1189.42D0/ ,MSIGN/1192.47D0/ ,MSIGM/1197.32D0/
*     CHARACTER * 4 INPS(4),INVCT(5),INSC(5),INDIF(4)
      DATA AMODEL/' MODEL-D ',' MODEL-F ',' NSC89   ',
     .            ' NSC97f  ',' NSC97e  ',
     .            ' ESC04A  ',' ESC04B  ',' ESC04C  ',' ESC04D  '/
      SAVE ICALL

C----------------------------------------------------------------------

      IF(ICALL.GT.0.and.IMODEL.EQ.IMODL) GOTO 100
      IMODL = IMODEL
***   if(ns.ne.0) open(17,file='outpot.data')

C     Defining model parameters:
      CALL MODPARS(IMODEL)
C     SPECIAL SETTINGS:
      IF(IGRTST.EQ.0) IGREEN=0
      IF(NCHAN.EQ.1) IDAM=0
      IF(IMODEL.LE.4.OR.IMODEL.GE.10) IDAM=0      ! check this 

      IF(ICALL.EQ.0) THEN
      WRITE(*,*) '======================================================
     .========'
      WRITE(N2,334) IMODEL,AMODEL(IMODEL),INRS,IRET,IDAM,IGERST,
     .IPSC,IVCT,ISCL,IDIF,IAXI,ITEN,
     .ITPS,IPAIR,IPIBE,IPSBE,
     .IZPS,IZVC,IZSC,IZAX,
     .IPV,IGM,INA,IOFF,NADIA,IRET,IGREEN
  334 FORMAT(/,' ***** MODEL INFORMATION: IMODEL=',I2,
     .' MODEL=',A9,' *****',//,
     .' INRS =',I2,' IRET =',I2,' IDAM =',I2,' IGERST=',I2,//,
     .' IPSC =',I2,' IVCT =',I2,' ISCL =',I2,' IDIF  =',I2,
     .' IAXI =',I2,' ITEN =',I2,/,
     .' ITPS =',I2,' IPAIR=',I2,' IPIBE=',I2,' IPSBE =',I2,/,
     .' IZPS =',I2,' IZVC =',I2,' IZSC =',I2,' IZAX  =',I2,//,
     .' IPV  =',I2,' IGM  =',I2,' INA  =',I2,' IOFF  =',I2,
     .' NADIA=',I2,//,' *** IRET=',I2,' IGREEN=',I2,' ***',/)
      WRITE(*,*) '======================================================
     .========'
      ENDIF
c-----------------------------------------------------------------------
C     SETING THE MESON-PARAMETERS
c-----------------------------------------------------------------------
c     yn-papers, yn obe hard-core-models:
      if(imodel.eq.1) open(9,file='tmp/parprd15.2004')
      if(imodel.eq.2) open(9,file='tmp/parprd20.2004')
c     yn-papers, yn obe soft-core-models:
      if(imodel.eq.3) open(9,file='tmp/parprc40.2004')
      if(imodel.eq.4) open(9,file='tmp/parnsc97f.2004')
      if(imodel.eq.5) open(9,file='tmp/parnsc97e.2004')    ! NOT YET READY
c     yn-paper, nn+yn models esc04a-d:
      if(imodel.eq.6) open(9,file='tmp/parbb.dec04.apv=0.5.05b10p')
      if(imodel.eq.7) open(9,file='tmp/parbb.esc04.izsc=1.f10')     
      if(imodel.eq.8) open(9,file='tmp/parbb.dec04.apv=0.5.05e10')
      if(imodel.eq.9) open(9,file='tmp/parbb.dec04.apv=1.0.05f10')
c-----------------------------------------------------------------------
      IF(IGREEN.NE.0) THEN
      WRITE(*,*) '++++++++++++++++++++++++++++++++++++++++++++++++++++++
     .++++++++'
      WRITE(*,*) ' !!! WARNING: IGREEN=',IGREEN,
     .           ' GREEN TRANSFORMATION, OKAY???? '
      WRITE(*,*) '++++++++++++++++++++++++++++++++++++++++++++++++++++++
     .++++++++'
      ENDIF

      IMODL = IMODEL
      IF(IMODEL.LE.2) NLOC=0
      IF(IMODEL.GE.3) NLOC=1
      AKS(1) = 0.D0
      AKS(2) = 0.D0
      AKS(3) = 0.D0
      AM(1) = 938.2592D0
      PIM   = 138.041D0
      AME   = 548.8D0
      AMK   = 495.8D0
      BMK   = 453.4D0
      AMX   = 957.5D0
      AMFI  = 1019.5D0
      AMOM  = 783.9D0
      AMKS  = 892.6D0
      BMKS  = 869.1D0
      PIM0  = PIM
C-------------------------------------------------------------
**    IF(NS.EQ.1) THEN
      WRITE(6,99)
      WRITE(6,599)
      WRITE(N2,10) AMODEL(IMODEL)
   10 FORMAT('0 THE PARAMETERS MODEL=',A9,/)
**    ENDIF
C-------------------------------------------------------------
      DO 21 I=1,20
      READ(9,*) (PAR(I,J),J=1,8)
      WRITE(6,22) I,(PAR(I,J),J=1,4)
      WRITE(6,22) I,(PAR(I,J),J=5,8)
   21 WRITE(6,*)
      REWIND 9
   22 FORMAT(' I=',I2,4(D15.7,2X))
      WRITE(6,99)
   99 FORMAT(72('-'))
C-------------------------------------------------------------
C
      XS  =PAR(1,1)
      XT  =PAR(2,1)
      X1P =PAR(3,1)
      X3P27=PAR(4,1)
      X3P8S=PAR(5,1)
      XHIGHL=X3P27    
C
C     TWO POLE APPROXIMATION UNSTABLE MESONS
C
      ARO  =PAR(4,4)
      AM1RO=PAR(4,3)
      BRO  =PAR(5,4)
      AM2RO=PAR(5,3)

      IF(IGERST.EQ.0) THEN
      ASI  =PAR(5,6)
      AM1SI=PAR(5,5)
      BSI  =PAR(6,6)
      AM2SI=PAR(6,5)
      ENDIF
C     BRYAN AND GERSTEN TWO-POLE PARAMETRIZATION BROAD EPSILON:
      IF(IGERST.EQ.1) THEN
      CALL GERSTPAR
      PAR(5,6) = ASI
      PAR(5,5) = AM1SI
      PAR(6,6) = BSI
      PAR(6,5) = AM2SI
      ENDIF
      AMD  =PAR(4,5)
      AMSST=PAR(7,5)
**    AMSCK=PAR(12,5)
      AMSCK=PAR(16,5)
C
C     DIFFRACTIVE MASSES
C
      AMA2 =PAR(8,6)
      AMPOM=PAR(9,6)
      AMF2 =AMPOM
**    AMKSS=PAR(8,6)
      AMKSS=PAR(16,6)
      AHYPC = PAR(11,6)          
C
C     NUCLEAR MEDIUM EFFECTS ON MESON MASSES:
C
      FACMV = 1D0
      FACMS = 1D0
      CALL PMEDIUM(FACMV,FACMS)

*     ICALL=1
100   IF(NCHNH.EQ.NCHAN.AND.LINTH.EQ.LINT.AND.JSPINH.EQ.JSPIN) GOTO 200
      NCHNH=NCHAN
      LINTH=LINT
      JSPINH=JSPIN
C *******************************************************************
C
C     FORM FACTOR MASSES :
C
C *******************************************************************
C     MODELS HC-D, HC-F:                     
      IF(IMODEL.LE.2) THEN
      ALAM = 10000.D0
      ALM1   = ALAM
      ALAM27 = ALAM
      ALAM8S = ALAM
      ALAM8A = ALAM
      ALAM10 = ALAM
      ALAM10S= ALAM
      IF(ICALL.EQ.0) WRITE(N2,201) ALM1,ALM27,ALM8S,ALM10,ALM8A,ALM10S
  201 FORMAT(/,' ALM1  =',F9.3,' ALM27 =',F9.3,' ALM8S =',F9.3,
     .       /,' ALM10 =',F9.3,' ALM8A =',F9.3,' ALM10S=',F9.3,//)
      ENDIF
C *******************************************************************
C     MODELS PRC40:                     
      IF(IMODEL.EQ.3) THEN
**    ALM1 =PAR(7,1)
      ALM1 =PAR(13,1)
      ALM27=PAR(8,1)
      ALM8S=PAR(9,1)
      ALM10=PAR(10,1)
      ALM8A=PAR(11,1)
      ALM10S=PAR(12,1)
      IF(NCHAN.NE.1 .AND. NCHAN.NE.4) ALM10=ALM10S
      IUSPIN=JSPIN+LINT+1
      IF(MOD(IUSPIN,2).EQ.0) ALAM1=ALM8A
      IF(MOD(IUSPIN,2).EQ.0) ALAM3=ALM10
      IF(MOD(IUSPIN,2).EQ.1) ALAM1=ALM8S
      IF(MOD(IUSPIN,2).EQ.1) ALAM3=ALM27
C     THE FOLLOWING GOOD FOR ALL LINT'S :
C     IF(NCHAN.EQ.1.AND.MOD(IUSPIN,2).EQ.0) ALAM=ALM10
C     IF(NCHAN.EQ.1.AND.MOD(IUSPIN,2).EQ.1) ALAM=ALM27
C     THE FOLLOWING ONLY GOOD FOR LINT=0 :
C     IF(NCHAN.EQ.1.AND.JSPIN.EQ.1) ALAM=ALM10
C     IF(NCHAN.EQ.1.AND.JSPIN.EQ.0) ALAM=ALM27
      IF(ICALL.EQ.0) WRITE(N2,202) ALM1,ALM27,ALM8S,ALM10,ALM8A,ALM10S
  202 FORMAT(/,' ALM1  =',F9.3,' ALM27 =',F9.3,' ALM8S =',F9.3,
     .       /,' ALM10 =',F9.3,' ALM8A =',F9.3,' ALM10S=',F9.3,//)
C
C     FOR THIS VERSION OF THE SOFTCORE OBE-MODEL:
C
      DO 5 I=1,6
      ALM(I)=ALAM1
      IF(I.GT.2) GOTO 5
      ALM(I+6)=ALAM3
   5  CONTINUE
      ENDIF
C *******************************************************************
C     SOFT-CORE 97-OBE-MODELs, and ESC-MODEL:
      IF(IMODEL.GE.4) THEN
      ALMP8 = PAR(8,2)
      ALMV8 = PAR(8,3)
      ALMS8 = PAR(8,5)
      ALMP1 = PAR(9,2)
      ALMV1 = PAR(9,3)
      ALMS1 = PAR(9,5)
**    ALAMK = PAR(12,2)
**    ALMKS = PAR(12,3)
**    ALMKAP= PAR(12,4)
      ALAMK = PAR(16,2)
      ALMKS = PAR(16,3)
      ALMKAP= PAR(16,4)
      ALMAX = PAR(9,7)
      IF(ALAMK.EQ.0.D0) ALAMK=ALMP8
      IF(ALMKS.EQ.0.D0) ALMKS=ALMV8
      IF(ALMKAP.EQ.0.D0) ALMKAP=ALMS8
      IF(ALMAX.EQ.0.D0) ALMAX=ALMP8
      ENDIF
C -------------------------------------------------------------------
C     MEDIUM EFFECTS FORM FACTORS:
C     not yet implemented in this program!!
C -------------------------------------------------------------------

      IF(ICALL.EQ.0) WRITE(N2,204) ALMP8,ALMV8,ALMS8,ALMP1,ALMV1,ALMS1,
     .              ALAMK,ALMKS,ALMKAP,ALMAX
  204 FORMAT(/,' ALMP8 =',F9.3,' ALMV8 =',F9.3,' ALMS8 =',F9.3,
     .       /,' ALMP1 =',F9.3,' ALMV1 =',F9.3,' ALMS1 =',F9.3,
     .       /,' ALAMK =',F9.3,' ALMKS =',F9.3,' ALMKAP=',F9.3,
     .       /,' ALMAX =',F9.3,//)
cc    ENDIF
C *******************************************************************

**    ICALL=1
      GOTO(1,2,3),NCHAN
   1  AMY(1)=MSIGP
      AMY(2)=MSIGP
      AMY(3)=MSIGP
      AM(1) =MPROT
      AM(2) =MPROT
      AM(3) =MPROT
      GOTO 4
   2  AMY(1)=MLAMB
      AMY(2)=MSIGN
      AMY(3)=MSIGM
      AM(1) =MNEUT
      AM(2) =MNEUT
      AM(3) =MPROT
      GOTO 4
   3  AMY(1)=MLAMB
      AMY(2)=MSIGP
      AMY(3)=MSIGN
      AM(1) =MPROT
      AM(2) =MNEUT
      AM(3) =MPROT
   4  CONTINUE

C     MEDIUM EFFECTS ON THE BARYON MASSES:
C     Yamamoto: M*(Lambda) < M(Lambda), M*(Sigma) > M(Sigma) !??
      FSTAR = 1D0  
**    FSTAR = 0.6D0     ! Ter Haar
      DO 13 KK=1,3
      AM(KK) = FSTAR * AM(KK)
  13  AMY(KK) = FSTAR * AMY(KK)
      WRITE(6,*) ' HNPOT04: MEDIUM EFFECT MASSES FSTAR=',FSTAR

      DO 12 I=1,3
      REDM(I)=AM(I)*AMY(I)/(AM(I)+AMY(I))
  12  TEM(I) =2*REDM(I)/(PIM**2)
      REDMM = (REDM(1)+REDM(2)+REDM(3))/3.D0
CC
CC    adaption to multi-channel S.E. as described in comments above :
CC
      AMLN=DSQRT(AMY(1)*AM(1))
      AMSN=DSQRT(AMY(2)*AM(1))
      AMH =0.5D0*(AMY(1)+AMY(2))
      AMSS=0.5D0*(AMY(2)+AMY(3))
***   AMLS=DSQRT(AMH*AM(1))
      AMNN=(AM(1)+AM(2)+AM(3))/3.D0
      AMLS=DSQRT(AMH*AMNN) 
C
 200  CALL WRLDCOP2(LINT,JSPIN,NS)

      MAN = MOD(LINT+JSPIN+1,2)
      IF(MAN.EQ.1) P=+1.D0
      IF(MAN.EQ.0) P=-1.D0
C
      IF(IGREEN.EQ.0) 
     .CALL VMATP(X,PIM,P,NCHAN,JSPIN,LINT,NLOC,NS)
      IF(IGREEN.EQ.1) 
     .CALL POTGR(X,PIM,P,NCHAN,JSPIN,LINT,NLOC,NS)
      IF(NBASE.EQ.1) GOTO 1000
C
c     IF(NS.GE.1.AND.ICALL.EQ.0)  PRINT 14,SYS(NCHAN),JSPIN,LINT
c 14  FORMAT(//,1X,A13,'-POTENTIALS ON THE PARTICLE-BASIS : JSPIN=',I2,
c    .            ' LINT=',I2,//)
c     IF(NCHAN.EQ.1.AND.ICALL.EQ.0) THEN
c     PRINT 15
c 15  FORMAT(1X,' : SEE POTENTIALS ON THE ISOSPIN BASIS')
c     GOTO 1000
c     ENDIF

C ---------------------------------------------------------------------
C     WRITING THE POTENTIALS:
C     NOMENCLATURE :
C     VPRT(1,*,*)= CENTRAL POTENTIAL , VPRT(2,*,*)= SPIN-SPIN POTENTIAL
C     VPRT(3,*,*)= TENSOR     ,,     , VPRT(4,*,*)= SPIN-ORBIT   ,,
C     VPRT(5,*,*)= QUADRATIC SPIN-ORBIT  POTENTIAL.
C     VPRT(6,*,*)= PHI               , VPRT(7,*,*)= PHI' ,
C     VPRT(8,*,*)= PHI''.
C
      DO 301 I=1,8
      BM(1,1)=VI(I,1)
      BM(2,2)=VI(I,2)
      BM(1,2)=VI(I,3)
      BM(3,3)=VI(I,4)
      BM(2,1)=BM(1,2)
      IF(NCHAN.EQ.2) CALL TRANLN(BMP,BM)
      IF(NCHAN.EQ.3) CALL TRANLP(BMP,BM)
      DO 302 II=1,3
      DO 302 JJ=1,3
 302  VPRT(I,II,JJ)=BMP(II,JJ)
C
      IF(NS.GE.3) THEN
      WRITE(N2,332) X,TYPV(I),(VPRT(I,1,JJ),JJ=1,3),
     .             (VPRT(I,2,JJ),JJ=1,3),(VPRT(I,3,JJ),JJ=1,3)
      ENDIF
 301  CONTINUE
 332  FORMAT(/,1X,' X=',F10.4,' PIM-1 ',2X,A16,2X,3(D12.5,3X),/,
     .      41X,3(D12.5,3X),/,41X,3(D12.5,3X))
C ---------------------------------------------------------------------
C
1000  ICALL=1
C
C ------------------------------------------------------------------
  599 FORMAT(72('*')/,'C'/,
     a'C     ESC-PARAMETERS :  PARAMETER-MATRIX'/,'C'/,
     b'C   PAR( 1,I)= XS     G1     FD1    FV1   GS1    GA2    FA1    
     .GSPAIR',/,
     c'C   PAR( 2,I)= XT     ALP    ALVD   ALVV  GS*    PSID   ALPA   
     .GVPAIR',/,
     d'C   PAR( 3,I)= X1P    GETAP  GOM    FOM   GEPS   GPOM   FE1    
     .FVPAIR',/,
     e'C   PAR( 4,I)= X3P27  PIONC  AM1RO  ARO   AMD    1.0    THA    
     .GPIRO1',/,
     f'C   PAR( 5,I)= XHIGL  PION0  AM2RO  BRO   AM1SI  ASI    GT1    
     .GPIRO0',/,
     g'C   PAR( 6,I)= PROTON ETA    AMOM   ELPHI AM2SI  BSI    GEFF   
     .GPISI ',/,
     h'C   PAR( 7,I)= NEUTRN AMX0   AMFI   PSIS1 AM1ST  1.0    FT1    
     .HOPAIR',/,
     i'C   PAR( 8,I)= ALM27  ALMP1  ALMV1  PSIS8 ALMS1  AMA2   FEFF   
     .GSISI ',/,
     j'C   PAR( 9,I)= ALM8S  ALMP0  ALMV0  THS1  ALMS0  AMPOM  ALMAX  
     .GPIET',/,  
     k'C   PAR(10,I)= ALM10  THPS   THV    THS2  AMEPS  THD    SONV   
     .GPIETP',/,
     l'C   PAR(11,I)= ALM8A  THPE   THRO   THDE  GAMEP  AHYPC  SONPV  
     .ALPAPR',/,     
     m'C   PAR(12,I)= FACNA  APV    FDNP1  FDNN1 GEPS2  UPOM   SONSC  
     .ALSCPR',/,
     n'C   PAR(13,I)= ---    HFPS1  HFD1   HFV1  ALS2   ---    SONVPR 
     .ALVDPR',/,
     o'C   PAR(14,I)= SON{1} FCHPI  HALVD  HALVV ---    ---    SONPPR 
     .ALVVPR',/, 
     p'C   PAR(15,I)= RSCAL  HFETP  HGOM   HFOM  ALS    GPOPP  SONSPR 
     .GPIPOM',/,                   
     q'C   PAR(16,I)= FACYN  ALMKA  ALMKS  ALMKP AMSCK  AMKSS  FB1    
     .GTENPR',/,
     r'C   PAR(17,I)= ---    ---    ---    ---   ---    ---    ALFBX  
     .FTENPR',/,
     s'C   PAR(18,I)= ---    ---    ---    ---   GPPS2  ---    FB9    
     .GPSPR ',/,
     t'C   PAR(19,I)= ---    ---    ---    ---   ---    ---    SONVA  
     .GPIROV',/,
     u'C   PAR(20,I)= AKMAX  ---    ---    ---   ---    ---    ---    
     .GPIOMV',/,'C',/,72('-'),/ )
C-----------------------------------------------------------------
      RETURN
      END
C ***************************************************************
      SUBROUTINE MODPARS(IMODEL)        
C ***************************************************************
C     iopt   = 4:   onebos and twomes parameters set in ynmain
C     imodel = 1:   Hard-core OBE-model-D (PRD15)     ! YN-model
C     imodel = 2:   Hard-core OBE-model-F (PRD20)     ! YN-model

C     imodel = 3:   Soft-core OBE-model   (PRC40)     ! YN-model
C     imodel = 4:   Soft-core OBE-model   NSC97f      ! YN-model
C     imodel = 5:   Soft-core OBE-model   NSC97e      ! YN-model

C     imodel = 6:   Soft-core ESC-model   ESC04A      !TRUE NN+YN-model
C     imodel = 6:   Soft-core ESC-model   ESC04B      !TRUE NN+YN-model
C     imodel = 6:   Soft-core ESC-model   ESC04C
C     imodel = 6:   Soft-core ESC-model   ESC04D      !TRUE NN+YN-model
C ***************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/MODEL/IMODL,INRS,ICNSTR,IRET,IDAM,IGERST,IGRTST
      COMMON/ONEBOS/IOBE(6)   
      COMMON/TWOMES/ITME(4)                  
      COMMON/ZEROS/IZERO(5)
      COMMON/PAROOM/APV,ISPEC(5)                
      DIMENSION INFO(23,9)    
C     in row 'imodel' INFO contains the model-parameters:
C     INFO: ipsc,ivct,iscl,idif,iax,iten,
C           ipair,itps,ipibe,ipsbe,
C           ipv,igm,ina,ioff,nadia, inrs,idam,igerst,igrtst
C           izps,izvc,izsc,izax       
      DATA INFO/1,1,1,0,0,0, 0,0,0,0, 0,0,0,0,0, 1,0,0,0, 0,0,0,0,
     .          1,1,1,0,0,0, 0,0,0,0, 1,1,0,0,0, 1,0,0,0, 0,0,0,0,
     .          1,1,1,1,0,0, 0,0,0,0, 1,0,0,0,0, 1,0,0,1, 0,0,0,0,
     .          1,1,1,1,0,0, 0,0,0,0, 1,0,0,0,0, 1,0,0,1, 0,0,0,0,
     .          1,1,1,1,0,0, 0,0,0,0, 1,0,0,0,0, 1,0,0,1, 0,0,0,0,
     .          1,1,1,1,1,0, 1,1,0,0, 1,0,1,1,0, 1,0,1,1, 0,0,1,1,
     .          1,1,1,1,1,0, 1,1,0,0, 1,0,1,1,0, 1,0,1,1, 0,0,1,1,
     .          1,1,1,1,1,0, 1,1,0,0, 1,0,1,1,0, 1,0,1,1, 0,0,1,1,
     .          1,1,1,1,1,0, 1,1,0,0, 1,0,1,1,0, 1,0,1,1, 0,0,1,1/

**    IF(IOPT.NE.4.AND.IOPT.NE.5.AND.IOPT.NE.6) THEN
      DO 1 KK=1,6
    1 IOBE (KK) = INFO(KK   ,IMODEL)
      DO 2 KK=1,4
    2 ITME (KK) = INFO(KK+6 ,IMODEL)
**    ENDIF
      DO 3 KK=1,5
    3 ISPEC(KK) = INFO(KK+10,IMODEL)
      INRS      = INFO(16   ,IMODEL)
      IDAM      = INFO(17   ,IMODEL)
      IGERST    = INFO(18   ,IMODEL)
      IGRTST    = INFO(19   ,IMODEL)
C     ZEROS IN FORM FACTOR:
      DO 4 KK=1,4
    4 IZERO(KK) = INFO(KK+19,IMODEL)
      RETURN
      END
C *******************************************************************
      SUBROUTINE PMEDIUM(FACMV,FACMS)                             
C *******************************************************************
C     POSSIBILITY INCLUSION NUCLEAR-MEDIUM EFFECTS ON MESON MASSES
C     LITERATURE: A.DF. JACKSON, Ann.Rev.Nucl.Sci. 33: 105-141 (1983)
C     MSIGMA(RHO_N) = MEPS*[ 1 - ALF*RHO_N + BET*RHO_N^5/3]
C -------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/ALLPS/AME,AMX,AMK,BMK
      COMMON/ALLVC/ARO,BRO,AM1RO,AM2RO,AMOM,AMFI,AMKS,BMKS
      COMMON/ALLSC/ASI,BSI,AM1SI,AM2SI,ASST,BSST,AMSST,AM2ST,AMD,AMSCK
      DATA RHON/0.18D0/, ALFN/0.5D0/
**    DATA FACMV/1D0/,FACMS/1D0/
      DATA MEDCALL/0/

cc    IF(MEDIUM.EQ.1) THEN
C  MEDIUM FACTORS:

**    facmv = 0.85d0
**    facms = 0.85d0

cc    FACMV = 1.D0-ALFN*RHON
cc    FACMS = 1.D0-ALFN*RHON
C     CHANGE IN VECTOR MESON MASSES:
      AM1RO = AM1RO*FACMV
      AM2RO = AM2RO*FACMV
      AMKS  = AMKS *FACMV
      BMKS  = BMKS *FACMV
      AMOM  = AMOM *FACMV
      AMFI  = AMFI *FACMV
C     CHANGE IN SCALAR MESON MASSES:
      AMD   = AMD  *FACMS
      AMSCK = AMSCK*FACMS
      AM1SI = AM1SI*FACMS
      AM2SI = AM2SI*FACMS
      AMSST = AMSST*FACMS
      AM2ST = AM2ST*FACMS
cc    ENDIF
      IF(MEDCALL.EQ.0) WRITE(6,*) ' IN PMEDIUM: FACMV=',FACMV,
     .' FACMS=',FACMS
      MEDCALL = MEDCALL + 1

      RETURN
      END
C ***************************************************************
*     BLOCK DATA YNBLK
C ***************************************************************
*     COMMON/INMES/INPS,INVCT,INSC,INDIF,INAX,INTEN
*     CHARACTER*4 INPS(4) /'YES','YES','YES','YES'/
*     CHARACTER*4 INVCT(4)/'YES','YES','YES','YES'/
*     CHARACTER*4 INSC(4) /'YES','YES','YES','YES'/
*     CHARACTER*4 INDIF(4)/'YES','YES','YES','YES'/
*     CHARACTER*4 INAX(4) /'NO ','NO ','NO ','NO '/
C     CHARACTER*4 INPS(4) /'NO ','NO ','NO ','NO '/
C     CHARACTER*4 INVCT(4)/'NO ','NO ','NO ','NO '/
C     CHARACTER*4 INSC(4) /'NO ','NO ','NO ','NO '/
C     CHARACTER*4 INDIF(4)/'NO ','NO ','NO ','NO '/
C     CHARACTER*4 INTEN(4)/'NO ','NO ','NO ','NO '/
*     END

C ***************************************************************
      SUBROUTINE WRLDCOP2(LINT,JSPIN,NS)
C ******************************************************
C     version september 2000:   version with 2 scalar nonets
C     
C     ioptsc = 2:
C     1) su3-nonet : eps(760), del(962), S*(993), kappa(900)
C     2) su3-nonet : f0(1370), a0(1450), f0(1580), k**(1430)

C     note: modern notation: eps9760)=f0(760), del(962)=a0(9800, S*(993)=f0(975)

C     used notations: f0(1370)=eps2, a0(1450)=del2, f091580)=S2, K**(1430)=kappa2
C ******************************************************************
C  Parameters: Organization DEC.2004
C ******************************************************************
C
C     OBE-PARAMETERS ETC.:
C
C   PAR( 1,I)= XS     G1     FD1    FV1   GS1    GA2    FA1    GSPAIR
C   PAR( 2,I)= XT     ALP    ALVD   ALVV  GS*    PSID   ALPA   GVPAIR
C   PAR( 3,I)= X1P    GETAP  GOM    FOM   GEPS   GPOM   FE1    FVPAIR
C   PAR( 4,I)= X3P27  PIONC  AM1RO  ARO   AMD    1.0    THA    GAPAIR
C   PAR( 5,I)= XHIGL  PION0  AM2RO  BRO   AM1SI  ASI    GT1    HAPAIR
C   PAR( 6,I)= PROTON ETA    AMOM   ELPHI AM2SI  BSI    GEFF   GPPAIR
C   PAR( 7,I)= NEUTRN AMX0   AMFI   PSIS1 AM1ST  1.0    FT1    HOPAIR
C   PAR( 8,I)= ALM27  ALMP1  ALMV1  PSIS8 ALMS1  AMA2   FEFF   GEPAIR
C   PAR( 9,I)= ALM8S  ALMP0  ALMV0  THS1  ALMS0  AMPOM  ALMAX  GPIET     
C   PAR(10,I)= ALM10  THPS   THV    THS2  AMEPS  THD    SONV   GPIETP
C   PAR(11,I)= ALM8A  THPE   THRO   THDE  GAMEP  AHYPC  SONPV  ALPAPR
C   PAR(12,I)= ALM10S APV    FDNP1  FDNN1 GEPS2  UPOM   SONSC  ALSCPR
C   PAR(13,I)= ALM1   HFPI   HFD1   HFV1  ALS2   ---    SONVPR ALVDPR
C   PAR(14,I)= ---    FCHPI  HALVD  HALVV ---    ---    SONPPR ALVVPR 
C   PAR(15,I)= RSCAL  HFETP  HGOM   HFOM  ALS    GPOPP  SONSPR GPIPOM

C   PAR(16,I)= ---    ALMKA  ALMKS  ALMKP AMSCK  AMKSS  FB1    GTENPR
C   PAR(17,I)= ---    ---    ---    ---   ---    ---    ALFBX  FTENPR
C   PAR(18,I)= ---    ---    ---    ---   GPPS2  ---    FB9    GPSPR
C   PAR(19,I)= ---    ---    ---    ---   ---    ---    SONVA  GPIROV
C   PAR(20,I)= ---    ---    ---    ---   ---    ---    ---    GPIOMV

C     HERE : PHYSICAL COUPLINGS, EXCEPT FOR
C            THE SU3-SINGLET COUPLINGS GPS1 AND FD1
C
C ******************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/MODEL/IMODEL,INRS,ICNSTR,IRET,IDAM,IGERST,IGRTST
      COMMON/ONEBOS/IPSC,IVCT,ISCL,IDIF,IAXI,ITEN
      COMMON/TWOMES/IPAIR,ITPS,IPIBE,IPSBE
      COMMON/PAROOM/APV,IPV,IGM,INA,IOFF,NADIA     
      COMMON/INMES/INPS,INVCT,INSC,INDIF,INAX,INTEN
      CHARACTER*4 INPS(4),INVCT(4),INSC(4),INDIF(4),INAX(4),INTEN(4)
      COMMON/PRMTRS/PAR(20,8)
      COMMON/MASSES/PIM,AM(3),AMY(3),REDM(3),PLAB,AK(3),AKS(3),TEM(3)
      COMMON/DYNMAS/REDMM,AMLN,AMSN,AMH,AMSS,AMLS,AMNN
      COMMON/FRMFAC/ALMOBE,ALMNN,ALMLL,ALMSS,ALMLS,ALMLK,ALMSK
      COMMON/FORMF/ALAM,ALMP8,ALMV8,ALMS8,ALMP1,ALMV1,ALMS1,
     .             ALAMK,ALMKS,ALMKAP
      COMMON/ALLPS/AME,AMX,AMK,BMK
      COMMON/ALLVC/ARO,BRO,AM1RO,AM2RO,AMOM,AMFI,AMKS,BMKS
      COMMON/ALLSC/ASI,BSI,AM1SI,AM2SI,ASST,BSST,AMSST,AM2ST,AMD,AMSCK
      COMMON/ALLDF/AMPOM,AMF2,AMA2,AMKSS,GDA(11)
      COMMON/ALLAX/ FA(11),FB(11),ALMAX                            
*     COMMON/MIXING/THPS,THV,THS,THD
      COMMON/COUCON/ALP,GX0,FX0,THPS,ALVD,GOM,THV,ALVV,FOM,THS,GDEL,
     .GEPS,GSST,ALF,GPOM,GPOMP,GA2,PSID,THD,AHYPC,ALS,SSCAL,ALD,ALPAX
      COMMON/ALLSC2/ELPHI,PSIS1,PSIS8,THSD1,THSD2,
     .              GSA(11),GSB(11),IOPTSC
      COMMON/HVY/HFPS(11),GT(11),FT(11),HFD(11),HFV(11)

      COMMON/ALLPR/GSPAIR,GVPAIR,FVPAIR,GAPAIR,HAPAIR,GPPAIR,HOPAIR
     .            ,GEPAIR,GPIET,GPIETP,GPIPOM
     .            ,ALPVPR,ALVDPR,ALVVPR,ALSCPR,ALPAPR

      COMMON/COUPL/ F1,F2,F3,F4,F5,F6,F7,F8,F9,F10,F11,
     .  G1,G2,G3,G4,G5,G6,G7,G8,G9,G10,G11,
     .  FD1,FD2,FD3,FD4,FD5,FD6,FD7,FD8,FD9,FD10,FD11,
     .  FV1,FV2,FV3,FV4,FV5,FV6,FV7,FV8,FV9,FV10,FV11,
     .  GS1,GS2,GS3,GS4,GS5,GS6,GS7,GS8,GS9,GS10,GS11,
     .  GD1,GD2,GD3,GD4,GD5,GD6,GD7,GD8,GD9,GD10,GD11,P
      DIMENSION F(11),G(11),FD(11),FV(11),GS(11),GD(11)
      EQUIVALENCE (F1,F(1)),(G1,G(1)),(FD1,FD(1)),(FV1,FV(1))
     * ,(GS1,GS(1)),(GD1,GD(1))
C
      DATA SR2/1.4142136D0/,SR3/1.732051D0/,                   
     ,     AMPRO/938.2796D0/,N2/6/,IEXDB/0/
     .     ALVVH/0.D0/,INIT/0/,ICALL/0/,I3P0/1/
C
C
C  F AND G  PSEUDOSCALAR
C  FD       VECTOR  DIRECT        = ELECTRIC
C  FV       VECTOR  DERIVATIVE    = MAGNETIC
C  USED INDEXING OF THE COUPLINGS :
C   1 =NNP       NNRO       NNDEL       NNA2
C   2 =SSP       SSRO       SSDEL       SSA2
C   3 =LSP       LSRO       LSDEL       LSA2
C   4 =LNK       LNKS       LNKAP       LNK**
C   5 =SNK       SNKS       SNKAP       SNK**
C   6 =NNE       NNOM       NNEPS       NNPOM
C   7 =LLE       LLOM       LLEPS       LLPOM
C   8 =SSE       SSOM       SSEPS       SSPOM
C   9 =NNX       NNFI       NNS*        NNF2
C   10=LLX       LLFI       LLS*        LLF2
C   11=SSX       SSFI       SSS*        SSF2
C
C  NNX STANDS FOR NNETA' ETC.
C  F AND G  PSEUDOSCALAR
C  FD       VECTOR  DIRECT COUPLING
C  FV       VECTOR  DERIVATIVE COUPLING
C
c     fstar = 0.6d0
c     if(icall.eq.0) ampro = ampro*fstar
**    APV   = PAR(15,2)
      APV   = PAR(12,2)
      SONV  = PAR(10,7)
      SONPV = PAR(11,7)
      SONSC = PAR(12,7)
      IF(SONPV.EQ.0.D0) SONPV=1.D0  
      IF(SONV.EQ.0.D0) SONV=SONPV
      IF(SONSC.EQ.0.D0) SONSC=SONPV
      IF(I3P0.EQ.1) THEN
**    SONPV= SONV
      SONSC= SONV
      SONVA= SONV
      ENDIF
      IF(ICALL.EQ.0) THEN
      WRITE(*,*) ' NOTICE: *************************************'
      WRITE(*,*) ' I3P0=',I3P0,' SONVP.NE.SONV=SONSC=SONVA'
***   WRITE(*,*) ' I3P0=',I3P0,' SONVP.EQ.SONV=SONSC=SONVA'
      WRITE(*,*) ' NOTICE: *************************************'
      ENDIF
C
C     SETTING VALUE EXCHANGE-OPERATOR P
C
      MAN=MOD(LINT+JSPIN+1,2)
      P=1.D0
      IF(MAN.EQ.0) P=-P    
C
  12  PI=DACOS(-1.D0)
      CONV=PI/180.D0
      RSCAL=DSQRT(PAR(15,1))
      IF(IMODEL.LE.2) THEN
      G1  =PAR(1,2)*RSCAL
      G9  =PAR(3,2)*RSCAL
      ENDIF
      IF(IMODEL.GE.3) THEN
      F1  =PAR(1,2)*RSCAL
      F9  =PAR(3,2)*RSCAL
      ENDIF
      ALP =PAR(2,2)
      THPS=PAR(10,2)
      FD1 =PAR(1,3)*RSCAL
      ALVD=PAR(2,3)
      GOM =PAR(3,3)*RSCAL
      THV =PAR(10,3)
      FV1 =PAR(1,4)*RSCAL
      ALVV=PAR(2,4)
      FOM =PAR(3,4)*RSCAL
      GDEL=PAR(1,5)*RSCAL
      GSST=PAR(2,5)*RSCAL
      GEPS=PAR(3,5)*RSCAL
      THS =PAR(10,4)
C
*     in this version only the 1-nonet model implemented:
C     two-nonet model parameters:
      ELPHI = PAR(6,4)
      PSIS1 = PAR(7,4)
      PSIS8 = PAR(8,4)
      THS1  = PAR(9,4)
      THS2  = PAR(10,4)

      GA2  =PAR(1,6)*RSCAL
      GPOM =PAR(3,6)*RSCAL
      PSID =PAR(2,6)
      THD  =PAR(10,6)
      AHYPC= PAR(11,6)

      FA1  = PAR(1,7)*RSCAL
      FE1  = PAR(3,7)*RSCAL
      THA  = PAR(4,7)
      ALPAX= PAR(2,7)
      IF(ALPAX.EQ.0.D0) ALPAX =ALP

      SON  = PAR(11,7)
C
      THPSD=THPS
      THVD =THV
      THSD =THS
      THDD =THD
      THAD =THA
      PSIDD=PSID
      THPS=THPS*CONV
      THV =THV *CONV
      THS =THS *CONV
      THD =THD *CONV
      THA =THA *CONV
      PSID=PSID*CONV
      ELPHI = ELPHI*CONV
      PSIS1 = PSIS1*CONV
      PSIS8 = PSIS8*CONV
      THS1  = THS1 *CONV
      THS2  = THS2 *CONV
C
C     SU3 FOR 1) PV-COUPLINGS PS-SCALAR MESONS
C             2) PAULI-COUPLINGS VECTOR MESONS
C*********************************************************************
C
C  PSEUDOSCALAR COUPLING CONSTANTS
C
C*********************************************************************
      IF(ICALL.EQ.0) WRITE(N2,501)
      IF(IMODEL.LE.2) THEN
      F1 =PIM*G1/(2.*AM(1))
      GX0= G9
      FX0=PIM*G9/(2.*AM(1))
      ENDIF
      IF(IMODEL.GT.2) THEN
      G1=2*AM(1)*F1/PIM
      FX0 = F9
      GX0=2*AM(1)*F9/PIM
      ENDIF
      COST=DCOS(THPS)
      SINT=DSIN(THPS)
      F9 = (FX0-SINT*(4*ALP-1.D0)*F1/SR3)/COST
      G9 = (GX0-SINT*(4*ALP-1.D0)*G1/SR3)/COST
      IF(IPV.EQ.0) CALL SU3(G,ALP)
      IF(IPV.EQ.1) CALL SU3(F,ALP)

*     IF(NS.GE.1) WRITE(N2,450) G9,F9,ALP,THPSD,IPV,APV,SONPV
      IF(ICALL.EQ.0) WRITE(N2,450) G9,F9,ALP,THPSD,IPV,APV,SONPV
 450  FORMAT(' PS-SCALAR SINGLET COUPLING G9= ',F12.6,' F9=',
     .F12.6,/,24X,' ALP=',F7.3,' THETA =',F12.6,'  IPV=',I2,
     ./,24X,' APV=',F7.3,' SONPV= ',F7.2,/)

C     SU(3)-SYMMETRY BREAKING:
c     IF(SONPV.NE.1.D0) THEN              
c     IF(IPV.EQ.0) CALL SU3SB(G,SONPV,1)
c     IF(IPV.EQ.1) CALL SU3SB(F,SONPV,1)
c     ENDIF
      IF(SONPV.NE.1D0) CALL SU3SB03(G,ALP,SONPV)
      IF(SONPV.NE.1D0) CALL SU3SB03(F,ALP,SONPV)
C
C     ETA - X0 MIXING
C
C     MIND DIFFERENCE IN POSITION WITH VECTOR AND SCALAR MESONS
C
 700  IF(IPV.EQ.1) THEN      
      DO 701 JO=6,8
      JS=JO+3
      WOV=F(JO)
      WSV=F(JS)
      F(JS)= COST*WSV+SINT*WOV
  701 F(JO)=-SINT*WSV+COST*WOV
      G1 =2*AM(1)*F1/PIM
      G2 =2*AMY(2)*F2/PIM
      G3 =2*AMH*F3/PIM
      G4 =2*AMLN*F4/PIM
      G5 =2*AMSN*F5/PIM
      G6 =2*AM(1)*F6/PIM
      G7 =2*AMY(1)*F7/PIM
      G8 =2*AMY(2)*F8/PIM
      G9 =2*AM(1)*F9/PIM
      G10=2*AMY(1)*F10/PIM
      G11=2*AMY(3)*F11/PIM
      ENDIF
      IF(IPV.EQ.0) THEN      
      DO 702 JO=6,8
      JS=JO+3
      WOV=G(JO)
      WSV=G(JS)
      G(JS)= COST*WSV+SINT*WOV
  702 G(JO)=-SINT*WSV+COST*WOV
      F1 =G1*PIM/(2*AM(1))       
      F2 =G2*PIM/(2*AMY(2))       
      F3 =G3*PIM/(2*AMH)       
      F4 =G4*PIM/(2*AMLN)       
      F5 =G5*PIM/(2*AMSN)       
      F6 =G6*PIM/(2*AM(1))        
      F7 =G7*PIM/(2*AMY(1))       
      F8 =G8*PIM/(2*AMY(2))       
      F9 =G9*PIM/(2*AM(1))       
      F10=G10*PIM/(2*AMY(1))        
      F11=G11*PIM/(2*AMY(3))        
      ENDIF
C
C     USED IN OBE-PROGRAMS (SCSP ETC.) TO INCLUDE MASS FACTORS IN
C     POTS: (OLD WAY OF HELDER!, HAS GIVEN CONFUSION IN THE PAST!)
*     F2=PIM*G2/(2.*AMY(2))
*     F3=PIM*G3/(2.*AMH)
*     F4=AMK*G4/(2.*AMLN)
*     F5=AMK*G5/(2.*AMSN)
*     F6=AME*G6/(2.0*AM(1))
*     F7=AME*G7/(2.*AMY(1))
*     F8=AME*G8/(2.*AMY(2))
*     F9=AMX*G9/(2.0*AM(1))
*     F10=AMX*G10/(2.0*AMY(1))
*     F11=AMX*G11/(2.0*AMY(3))
C
C  !! IN THIS VERSION: WE INCLUDE THESE IN ROUTINE PSSCAL!!

C  SELECTION PSEUDO-SCALAR CONTRIBUTIONS
      CALL SELECT(F,INPS)
      CALL SELECT(G,INPS)
C*********************************************************************
C
C  COUPLING CONSTANTS FOR  VECTOR MESON POTENTIALS
C
C*********************************************************************
      IF(ICALL.EQ.0) WRITE(N2,501)
      FD(9)=GOM
      FV(9)=FOM
      COST=DCOS(THV)
      SINT=DSIN(THV)
      IF(IGM.EQ.0) THEN
      FD9 = (GOM-SINT*(4*ALVD-1.D0)*FD1/SR3)/COST
      FV9 = (FOM-SINT*(4*ALVV-1.D0)*FV1/SR3)/COST
      ENDIF
      IF(IGM.EQ.1) THEN
      FD9 = (GOM-SINT*(4*ALVD-1.D0)*FD1/SR3)/COST
      FV9 = (GOM+FOM-SINT*(4*ALVV-1.D0)*(FD1+FV1)/SR3)/COST
      ENDIF
      CALL SU3(FD,ALVD)
      CALL SU3(FV,ALVV)
*     IF(NS.GE.1) WRITE(N2,550) FD9,FV9,ALVD,ALVV,THVD,IGM
      IF(ICALL.EQ.0) WRITE(N2,550) FD9,FV9,ALVD,ALVV,THVD,IGM
     .,INRS,SONV
 550  FORMAT(' VECTOR SINGLET COUPLINGS FD9= ',F12.6,' FV9=',
     .F12.6,/,24X,' ALVD=',F12.6,' ALVV=',F12.6,' THETA=',F12.6,
     ./,24X,' IGM=',I2,' INRS=',I2,' SONV=',F7.2,//)

C     SU(3)-SYMMETRY BREAKING:
c     IF(SONV.NE.1.D0) CALL SU3SB(FD,SONV,2)
c     IF(SONV.NE.1.D0) CALL SU3SB(FV,SONV,2)
      IF(SONPV.NE.1D0) CALL SU3SB03(FD,ALVD,SONV)
      IF(SONPV.NE.1D0) CALL SU3SB03(FV,ALVV,SONV)
      ALVVH=ALVV
      INIT=1
      DO 5 JO=6,8
      JS=JO+3
      WOV=FV(JO)
      WOD=FD(JO)
      WSV=FV(JS)
      WSD=FD(JS)
      FV(JO)= COST*WSV+SINT*WOV
      FD(JO)= COST*WSD+SINT*WOD
      FV(JS)=-SINT*WSV+COST*WOV
      FD(JS)=-SINT*WSD+COST*WOD
    5 CONTINUE
      IF (IGM.EQ.0) GO TO 711
      DO 712 I=1,11
  712 FV(I)=FV(I)-FD(I)
  711 CONTINUE

C  SELECTION VECTOR MESON CONTRIBUTIONS
      CALL SELECT(FD,INVCT)
      CALL SELECT(FV,INVCT)
C*********************************************************************
C
C     SCALAR COUPLING CONSTANTS
C
C*********************************************************************
      IF(ICALL.EQ.0) WRITE(N2,501)
  501 FORMAT(72('-'))
      IOPTSC=2
      GEPS2 = PAR(13,6)*RSCAL
      GSST2 = 0.D0

C     SEPARATION TWO NONETS: DEL(962) AND DEL2(1450) NN-COUPLINGS:
      GDELI = GDEL
      GDELI2= 0.D0
      PSIS8D= PSIS8/CONV
      CALL CNSTR97(PSIS8D,GDELI,GDELI2,GDEL,GDEL2,2)        

C     REDEFINITION EPS(760) AND GEPS(1370) NN-COUPLINGS:
      GEPSI = GEPS 
      GEPS2I= GEPS2
      ELPHI2= PSIS1/CONV
      CALL CNSTR97(ELPHI2,GEPSI,GEPS2I,GEPS,GEPS2,5)

C     REDEFINITION EPS(760) AND S(993) NN-COUPLINGS:
      GEPSI = GEPS
      GSSTI = GSST
      ELPHID= ELPHI/CONV
      CALL CNSTR97(ELPHID,GEPSI,GSSTI,GEPS,GSST,1)

C     REDEFINITION EPS(1370) AND S(1580) NN-COUPLINGS:
c     GEPSI = GEPS2
c     GSSTI = GSST2
c     ELPHI2= PSIS1/CONV
c     CALL CNSTR97(ELPHI2,GEPSI,GSSTI,GEPS2,GSST2,3)
C---------------------------------------------------------------------
C     FIRST SCALAR NONET:
      GS1=GDEL
      COST=DCOS(THS)
      SINT=DSIN(THS)
C**   SU3-assignments 'bare' mesons:
C     warning: check consistency with SCALAR !!
C**   S* similar to PHI(1020):
      GS9 = COST*GEPS-SINT*GSST
      GS6 = SINT*GEPS+COST*GSST
C     ABANDOING CONSTRAINT: ALS=ALS(THS1):
      ALS = PAR(15,5)
      IF(PAR(15,5).NE.0.D0) THEN
      GS(9)= (GEPS-SINT*(4*ALS-1.D0)*GS(1)/SR3)/COST
      GSST = -SINT*GS(9)+COST*(4*ALS-1.D0)*GS(1)/SR3
      ENDIF
      IF(ALS.EQ.0.D0.AND.GS1.NE.0.D0) 
     .ALS = 0.25D0*(SR3*GS6/GS1+1.D0)
      IF(ALS.EQ.0.AND.DABS(GS1).LT.1.D-06) ALS = 0.25D0

      CALL SU3(GS,ALS)

C     SU(3)-SYMMETRY BREAKING:
c     IF(SONSC.NE.1.D0) CALL SU3SB(GS,SONSC,3)
      IF(SONSC.NE.1D0) CALL SU3SB03(GS,ALS,SONSC)

      THSD = THS/CONV
      IF(ICALL.EQ.0) THEN
      WRITE(N2,500) IOPTSC,GS9,GEPS,ALS,THSD,SONSC
     .,PAR(16,5),ALMKAP    
 500  FORMAT(' 1rst SCALAR NONET SU3{1}: IOPTSC=',I2,' GS9= ',F9.5,
     .' GEPS =',F9.5,/,22X,' ALS =',F9.3,' THS =',F9.3,                   
     .' SONSC=',F6.2,/,22X,' AMSCK=',F8.3,' ALMKAP=',F8.3,/)
      ENDIF

      DO 55 JO=6,8
      JS=JO+3
      WOS=GS(JO)
      WSS=GS(JS)
      GS(JO)= COST*WSS+SINT*WOS
   55 GS(JS)=-SINT*WSS+COST*WOS

C  SELECTION SCALAR MESON CONTRIBUTIONS
      CALL SELECT(GS,INSC)

C     SU3-NONET COUPLINGS STORED IN GSB(KK):
      DO 56 KK=1,11
   56 GSB(KK) = GS(KK)
C---------------------------------------------------------------------
C     SECOND SCALAR NONET:
      GSA(1)=GDEL2 
      COST=DCOS(THS1)
      SINT=DSIN(THS1)
C**   SU3-assignments 'bare' mesons:
C     warning: check consistency with SCALAR !!
C**   S* similar to PHI(1020):
      GSA(9) = COST*GEPS2-SINT*GSST2
      GSA(6) = SINT*GEPS2+COST*GSST2

      IF(GSA(1).NE.0.D0) ALS2 = 0.25D0*(SR3*GSA(6)/GSA(1)+1.D0)
      IF(DABS(GSA(1)).LT.1.D-06) ALS2 = 0.25D0
      IF(PAR(13,5).NE.0.D0) ALS2=PAR(13,5)

      CALL SU3(GSA,ALS2)

      IF(ICALL.EQ.0) THEN
      THSD1 = THS1/CONV
      WRITE(N2,511) IOPTSC,GSA(9),GEPS2,ALS2,THSD1          
 511  FORMAT(' 2nd SCALAR NONET SU3{1}: IOPTSC=',I2,' GS9= ',F9.5,
     .' GEPS2=',F9.5,/,22X,' ALS2=',F9.3,' THS1=',F9.3)                   
      ENDIF                 

      DO 57 JO=6,8
      JS=JO+3
      WOS=GSA(JO)
      WSS=GSA(JS)
      GSA(JO)= COST*WSS+SINT*WOS
   57 GSA(JS)=-SINT*WSS+COST*WOS

C  SELECTION SCALAR MESON CONTRIBUTIONS
      CALL SELECT(GSA,INSC)
C---------------------------------------------------------------------
      IF(ICALL.EQ.0)
     .WRITE(N2,512) IGERST,PAR(10,5),PAR(11,5),ASI,AM1SI,BSI,AM2SI
 512  FORMAT(/,72('-'),/,' TREATMENT BROAD EPSILON: IGERST=',I2,
     .' AMEPS=',F9.3,' GAMEP=',F9.3,//,2X,' ASI=',F9.5,' AM1SI=',F12.5,
     .' BSI=',F9.5,' AM2SI=',F12.5,/)

C*********************************************************************
C
C     POMERON-TYPE OF COUPLINGS
C
C*********************************************************************
C     VERSION SEPTEMBER 2004
C*********************************************************************
      IF(ICALL.EQ.0) WRITE(N2,501)
      IF(GA2.EQ.0D0) THEN
      PSID = 0D0
      DO 113 KK=1,11
  113 GDA(KK) = 0D0
      ENDIF
      GPOM1 = GPOM*DCOS(PSID)
      GPOMP = GPOM*DSIN(PSID)
      GD(1) = 0D0
      GD(9) = GPOM1
      CALL SU3(GD,ALVD)
      PSIDD=PSID/CONV
      IF(ICALL.EQ.0) WRITE(N2,400) GPOM,GPOM1,PSIDD          
 400  FORMAT(' POMERON SINGLET COUPLING GPOM = ',F9.5,
     ,' GPOM1 =',F9.5,' PSID=',F9.3,/)

      IF(GA2.NE.0D0) THEN
      GDA(1) = GA2
      ALD=ALVD
      SOND = SONV
      THD =THV
      THDD=THD/CONV
      SINT=DSIN(THD)
      COST=DCOS(THD)
      GDA(9) = (GPOMP-SINT*(4*ALD-1D0)*GDA(1)/SR3)/COST 
      IF(ICALL.EQ.0) WRITE(N2,401) GDA(1),GDA(9),ALD,THDD,PSIDD,AHYPC
 401  FORMAT(' DIFFRACTIVE NONET GDA1 = ',F9.5,' GDA9=',F9.5,
     .' ALPHA =',F9.3,/,24X,' THETA =',F12.6,' PSID=',F10.4,
     .' AHYPC=',F10.4,/)
      CALL SU3(GDA,ALD)
      IF(SOND.NE.1D0) CALL SU3SB03(GDA,ALD,SOND)

      DO 65 JO=6,8
      JS=JO+3
      WOD=GDA(JO)
      WSD=GDA(JS)
      GDA(JO)= COST*WSD+SINT*WOD
   65 GDA(JS)=-SINT*WSD+COST*WOD
      ENDIF
C     FLAVOR INDEPENDENCE : MASS-FACTORS
c     IF(AHYPC.EQ.0.D0) GOTO 64
c     GD2=GD2*AMPRO/AMSS
c     GD3=GD3*AMPRO/DSQRT(AMSS*AMY(1))
c     GD4=GD4*AMPRO/DSQRT(AMLN*AM(1) )
c     GD5=GD5*AMPRO/AMLS
c     GD7=GD7*AMPRO/AMY(1)
c     GD8=GD8*AMPRO/AMSS
c     GD10=GD10*AMPRO/AMY(1)
c     GD11=GD11*AMPRO/AMSS
c  64 CONTINUE

C  SELECTION DIFFRACTIVE CONTRIBUTIONS
      CALL SELECT(GD,INDIF)
      IF(ICALL.EQ.0) WRITE(N2,501)
C*********************************************************************
C
C     AXIAL COUPLING CONSTANTS
C
C*********************************************************************
      IF(IAXI.EQ.0) THEN
      FA1   = 0D0
      FE1   = 0D0
      ENDIF
      FA(1) = FA1     
      ALMAX = PAR(9,7)
      COST  = DCOS(THA)
      SINT  = DSIN(THA)
      FA(9) = (FE1-SINT*(4*ALPAX-1.D0)*FA1/SR3)/COST
      CALL SU3(FA,ALPAX)
      IF(SONVA.NE.1D0) CALL SU3SB03(FA,ALPAX,SONVA)
      IF(ICALL.EQ.0) THEN
      THAD  = THA/CONV
      WRITE(N2,514) FA(1),FA(9),ALPAX,THAD,ALMAX     
 514  FORMAT(' AXIAL NONET SU3{1}: FA1  =',F9.5,' FA9=',F9.5,
     ./,20X,' ALPAX=',F9.3,' THA=',F9.3,' ALMAX=',F10.3)                   
      ENDIF                 
      DO 75 JO=6,8
      JS=JO+3
      WOS=FA(JO)
      WSS=FA(JS)
      FA(JO)= COST*WSS+SINT*WOS
   75 FA(JS)=-SINT*WSS+COST*WOS

C  SELECTION AXIAL MESON CONTRIBUTIONS
      CALL SELECT(FA,INAX)
C*********************************************************************
      IF(IAXI.EQ.0) THEN
      FB(1) = 0D0
      FB9   = 0D0
      ENDIF
      IF(IAXI.NE.0.AND.IEXDB.EQ.0) THEN
**    FB(1) = PAR(13,3)
**    FB(9) = PAR(15,3)
      FB(1) = PAR(16,7)
      FB(9) = PAR(18,7)
      ENDIF
      IF(IAXI.NE.0.AND.IEXDB.EQ.1) THEN
      SRB = 10D0*AMPRO**2
      FEXD = 1D0/DSQRT(SRB/AMPRO**2)
      FB(1) = FEXD* F1*2*AMPRO/PIM  *RSCAL
      FB9   = FEXD* FX0*2*AMPRO/PIM  *RSCAL
      ENDIF
**    ALFBX = PAR(14,3)
      ALFBX = PAR(17,7)
      IF(ALFBX.EQ.0D0) ALFBX = ALP
      THB = THPS
      COST  = DCOS(THB)
      SINT  = DSIN(THB)
      FB(9) = (FB9-SINT*(4*ALFBX-1.D0)*FB(1)/SR3)/COST
      CALL SU3(FB,ALFBX)
      IF(SONPV.NE.1D0) CALL SU3SB03(FB,ALFBX,SONVP)
      THBD  = THB/CONV
      IF(ICALL.EQ.0) WRITE(N2,516) FB(1),FB(9),ALFBX,THBD           
 516  FORMAT(' B-AXIAL NONET SU3{1}: FB1  =',F9.5,' FB9=',F9.5,
     ./,20X,' ALFBX=',F9.3,' THb=',F9.3,/)                                 
      DO 85 JO=6,8
      JS=JO+3
      WOS=FB(JO)
      WSS=FB(JS)
      FB(JO)= COST*WSS+SINT*WOS
   85 FB(JS)=-SINT*WSS+COST*WOS

C  SELECTION AXIAL MESON CONTRIBUTIONS
      CALL SELECT(FB,INAX)
C*********************************************************************
C*********************************************************************
C
C     TENSOR MESON COUPLINGS 
C
C*********************************************************************
      IF(ITEN.EQ.0) THEN
      GT(1) = 0D0
      FT(1) = 0D0
      GEFF  = 0D0
      FEFF  = 0D0
      ENDIF       

      IF(ICALL.EQ.0) WRITE(N2,501)
      THT  = THV
      ALTD = ALVD
      ALTV = ALVV
      IF(ITEN.NE.0) THEN
      GT(1)= PAR(5,7)
      GEFF = PAR(6,7)
      FT(1)= PAR(7,7)
      FEFF = PAR(8,7)
c     vector-meson values:
      gt(1)= par(1,3)
      ft(1)= par(1,4)
      geff = gom
      feff = fom
**    altd = 0d0
      ENDIF
      COST=DCOS(THT)
      SINT=DSIN(THT)
      GT(9) = (GEFF-SINT*(4*ALTD-1.D0)*GT(1)/SR3)/COST
      FT(9) = (FEFF-SINT*(4*ALTV-1.D0)*FT(1)/SR3)/COST
      CALL SU3(GT,ALTD)
      CALL SU3(FT,ALTV)
      IF(ICALL.EQ.0) WRITE(N2,900) GT(9),FT(9),ALTD,ALTV,
     .THT/CONV
 900  FORMAT(' TENSOR SINGLET COUPLINGS GT9= ',F12.6,' FT9=',
     .F12.6,/,24X,' ALTD=',F12.6,' ALTV=',F12.6,' THETA=',F12.6,//)

C     SU(3)-SYMMETRY BREAKING:
c     IF(SONV.NE.1.D0) CALL SU3SB(GT,SONV,2)
c     IF(SONV.NE.1.D0) CALL SU3SB(FT,SONV,2)
      DO 95 JO=6,8
      JS=JO+3
      WOV=FT(JO)
      WOD=GT(JO)
      WSV=FT(JS)
      WSD=GT(JS)
      FT(JO)= COST*WSV+SINT*WOV
      GT(JO)= COST*WSD+SINT*WOD
      FT(JS)=-SINT*WSV+COST*WOV
      GT(JS)=-SINT*WSD+COST*WOD
   95 CONTINUE

C  SELECTION TENSOR MESON CONTRIBUTIONS
      CALL SELECT(GT,INTEN)
      CALL SELECT(FT,INTEN)
C*********************************************************************
C
C*********************************************************************
      IF(ICALL.EQ.0) THEN
      WRITE(N2,501)
      WRITE(N2,93) G,F,FD,FV,GSB,GSA,GD,GDA,FA,FB,GT,FT
      ENDIF
   93 FORMAT(/,'  COUPLING CONSTANTS : ',//,
     . 7x,'   nnp  ','   ssp  ','   slp  ','   lnk  ','   snk  ',/,
     . 7x,'   nne  ','   lle  ','   sse  ','   nnx  ','   llx  ',
     .    '   ssx  ',//,
     1 '  gps =',5f8.4,/,7x,6f8.4,//,'  fps =',5f8.4,/,7x,6f8.4,//,     
     2 '   fd =',5f8.4,/,7x,6f8.4,//,'   fv =',5f8.4,/,7x,6f8.4,//,     
     3 '  gsb =',5f8.4,/,7x,6f8.4,//,'  gsa =',5f8.4,/,7x,6f8.4,//,     
     4 '   gd =',5f8.4,/,7x,6f8.4,//,'  gda =',5f8.4,/,7x,6f8.4,//,     
     5 '   fa =',5f8.4,/,7x,6f8.4,//,'   fb =',5f8.4,/,7x,6f8.4,//,     
     6 '   gt =',5f8.4,/,7x,6f8.4,//,'   ft =',5f8.4,/,7x,6f8.4,//)     
C
C     PAIR COUPLINGS
C
      IF(IPAIR.NE.0) THEN
      GSPAIR = PAR(1,8)*RSCAL
      GVPAIR = PAR(2,8)*RSCAL
      FVPAIR = PAR(3,8)*RSCAL
      GAPAIR = PAR(4,8)*RSCAL
      HAPAIR = PAR(5,8)*RSCAL
      GPPAIR = PAR(6,8)*RSCAL
      HOPAIR = PAR(7,8)*RSCAL
      GEPAIR = PAR(8,8)*RSCAL
      GPIET  = PAR(9,8)*RSCAL
      GPIETP = PAR(10,8)*RSCAL
      GPIPOM = PAR(15,8)*RSCAL
C     GSPAIR = 0.D0
C     GVPAIR = 0.D0
C     FVPAIR = 0.D0
C     GAPAIR = 0.D0
C     HAPAIR = 0.D0
C     GPPAIR = 0.D0
C     HOPAIR = 0.D0
C     GEPAIR = 0.D0
C     GPIET  = 0.D0
C     GPIETP = 0.D0
      ALPVPR = ALP
*     ALVDPR = ALVD
*     ALVVPR = ALVV
*     ALSCPR = ALS
      ALVDPR = PAR(13,8)
      ALVVPR = PAR(14,8)
      ALSCPR = PAR(12,8)
      ALPAPR = PAR(11,8)
**    ALPAPR = ALPAX

      CALL YNPRCOP
      ENDIF

      ICALL = 1
      RETURN
      END
C ***************************************************************
      SUBROUTINE SELECT(F,INMES)         
C ***************************************************************
C     CHECKING CONTRIBUTIONS INDIVIDUAL MESONS
C***********************************************************************
C  USED INDEXING OF THE COUPLINGS :
C   1 =NNP       NNRO       NNDEL       NNA2
C   2 =SSP       SSRO       SSDEL       SSA2
C   3 =LSP       LSRO       LSDEL       LSA2
C   4 =LNK       LNKS       LNKAP       LNK**
C   5 =SNK       SNKS       SNKAP       SNK**
C   6 =NNE       NNOM       NNEPS       NNPOM
C   7 =LLE       LLOM       LLEPS       LLPOM
C   8 =SSE       SSOM       SSEPS       SSPOM
C   9 =NNX       NNFI       NNS*        NNF2
C   10=LLX       LLFI       LLS*        LLF2
C   11=SSX       SSFI       SSS*        SSF2
C
C ***************************************************************
C  NNX STANDS FOR NNETA' ETC.
C ***************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER * 4 INMES(4)
      DIMENSION F(11)
      IF(INMES(1).EQ.'NO ') THEN 
      F(1) = 0.D0
      F(2) = 0.D0
      F(3) = 0.D0
      ENDIF
      IF(INMES(2).EQ.'NO ') THEN 
      F(6) = 0.D0
      F(7) = 0.D0
      F(8) = 0.D0
      ENDIF
      IF(INMES(3).EQ.'NO ') THEN 
      F(9) = 0.D0
      F(10) = 0.D0
      F(11) = 0.D0
      ENDIF
      IF(INMES(4).EQ.'NO ') THEN 
      F(4) = 0.D0
      F(5) = 0.D0
      ENDIF
      RETURN
      END
C ***************************************************************
      SUBROUTINE SU3(F,ALP)
C ***************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION F(11)
      DATA SR3/1.732051D0/
      FF   =  F(1)
      F(2) =2*FF*ALP
      F(3) =2*FF*(1.D0-ALP)/SR3
      F(4) = -FF*(1.D0+2*ALP)/SR3
      F(5) =  FF*(1.D0-2*ALP)
      F(6) =  FF*(4*ALP-1.D0)/SR3
      F(7) = -F(3)
      F(8) =  F(3)
      F(10)=  F(9)
      F(11)=  F(9)
      RETURN
      END
C **********************************************************************
      SUBROUTINE SU3SB(F,SONA,MTYP)
C **********************************************************************
C     MTYP=1: PSEUDO-SCALAR, MTYP=2: VECTOR, MTYP=3: SCALAR,DIFRAC
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION F(11),MEX(3)
      DATA MEX/1,1,0/,ICALL/0/
      IW = MEX(MTYP)
***   SON = SONA*SONA
      SON = DABS(SONA)     
      F(4) = F(4) - F(4)*(1.D0-SON)
      F(5) = F(5) - F(5)*(1.D0-SON)
      IF(IW.EQ.1) THEN
      F(7) = F(7) 
      F(8) = F(8) - F(8)*(1.D0-SON*SON)/3.D0
      F(10)= F(10)- F(9)*(1.D0-SON*SON)
      F(11)= F(11)+ F(9)*(1.D0-SON*SON)/3.D0
      ENDIF
      IF(IW.EQ.0) THEN
      F(7) = F(7) + F(1)*(1.D0-SON*SON)*2/DSQRT(3.D0)
      F(8) = F(8) + F(1)*(1.D0-SON*SON)*2/DSQRT(3.D0)
      F(10)= F(10)- F(9)*(1.D0-SON*SON)/3.D0
      F(11)= F(11)- F(9)*(1.D0-SON*SON)/3.D0
      ENDIF
      IF(ICALL.EQ.0) THEN
      WRITE(*,99)
      WRITE(*,*) ' SU3SB: IW(PV)=',MEX(1),' IW(VCT)=',MEX(2),
     .' IW(SCL)=',MEX(3)
      WRITE(*,99)
   99 FORMAT(/,72('-'),/)
      ICALL=1
      ENDIF
      RETURN
      END


C **********************************************************************
cc    SUBROUTINE SU3SB(F,SON)
C **********************************************************************
cc    IMPLICIT REAL*8(A-H,O-Z)
cc    DIMENSION F(11)
cc    F(4) = F(4) - F(4)*(1.D0-SON)
cc    F(5) = F(5) - F(5)*(1.D0-SON)
cc    F(7) = F(7) 
cc    F(8) = F(8) - F(8)*(1.D0-SON*SON)/3.D0
cc    F(10)= F(10)- F(9)*(1.D0-SON*SON)
cc    F(11)= F(11)+ F(9)*(1.D0-SON*SON)/3.D0
cc    RETURN
cc    END
C ***************************************************************
*     SUBROUTINE SU3SB(F,SON)
C ***************************************************************
*     IMPLICIT REAL*8(A-H,O-Z)
*     DIMENSION F(11)
*     DATA SR2/1.414213562D0/,SR3/1.732051D0/
*     F(4) =  F(4)-F(4)*(1.D0-SON)
*     F(5) =  F(5)-F(5)*(1.D0-SON)
*     DELSB=  F(1) * (1.D0-SON*SON)*SR2/SR3
*     F(7) =  F(7) + SR2*DELSB
*     F(8) =  F(8) + SR2*DELSB
*     F(10)=  F(10)- DELSB
*     F(11)=  F(11)- DELSB
*     RETURN
*     END
C *****************************************************************
      SUBROUTINE VMATP(X,PIM,P,NCHAN,JSPIN,LINT,NLOC,NS)
C *****************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/ONEBOS/IPSC,IVCT,ISCL,IDIF,IAXI,ITEN
      COMMON/TWOMES/IPAIR,ITPS,IPIBE,IPSBE
      COMMON/PAROOM/APV,IPV,IGM,INA,IOFF,NADIA     
      COMMON/FORMF/ALAM,ALMP8,ALMV8,ALMS8,ALMP1,ALMV1,ALMS1,
     .             ALAMK,ALMKS,ALMKAP
      COMMON/POT/VCLL,VSIGLL,VTENLL,VSOLL,VSO2LL,FILL,DFILL,DDFILL,
     .           VCSS,VSIGSS,VTENSS,VSOSS,VSO2SS,FISS,DFISS,DDFISS,
     .           VCLS,VSIGLS,VTENLS,VSOLS,VSO2LS,FILS,DFILS,DDFILS,
     .           VCDR,VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR,
     .           FISLL,DFISLL,DDFSLL,FISSS,DFISSS,DDFSSS,
     .           FISLS,DFISLS,DDFSLS,FISDR,DFISDR,DDFSDR,
     .           VASOLL,VASOSS,VASOLS,VASODR
**   .          ,VCNN,VSIGNN,VTENNN,VSONN,VSO2NN
      COMMON/POTF/VCAA(4),VSIGAA(4),VTENAA(4),VSOAA(4),
     .            VSO2AA(4),VASOAA(4),VNNAA(4),
     .            FIAA(4),DFIAA(4),              
     .            FISAA(4),DFISAA(4),             
     .            FITAA(4),DFITAA(4)             
      COMMON/POTESC/VCESC(4),VSESC(4),VTESC(4),VOESC(4),VAESC(4)
      COMMON/PNNESC/ESCCNN,ESCSNN,ESCTNN,ESCONN
      CHARACTER * 16 SYS(3)
      CHARACTER *4 TYPM(8),TYPV(8),CHAN(4),TYPV2(8)
      DATA SYS/'SIGMA(+)-PROTON','LAMBDA-NEUTRON','LAMBDA-PROTON'/
      DATA TYPM/'PSSC','VCTR','SCAL','DIFR','PSPS','PAIR','PIBE',     
     .     'PSBE'/, CHAN/'LL','SS','LS','DR'/,
     .     TYPV/' VC ','VSIG','VTEN',' VSO','VASO',' FI ',' FIS','DDFI'/
     .    TYPV2/' VC ','VSIG','VTEN',' FI ',' FIS','DFI ','DFIS','DDFI'/
      DATA ZERO/0.D0/,N2/6/,ICALL/0/,ICALLS/0/,ICALLT/0/
C
      ICALL=ICALL+1
      IF(JSPIN.EQ.0) ICALLS=ICALLS+1
      IF(JSPIN.EQ.1) ICALLT=ICALLT+1
      IF(ICALL.EQ.1.AND.NS.GE.1)  PRINT 13,SYS(NCHAN),JSPIN,LINT
  13  FORMAT(//,1X,A13,'-POTENTIALS ON THE ISOSPIN-BASIS : JSPIN=',I2,
     .            ' LINT=',I2,//)
      ICSB=1
      IF(NCHAN.EQ.2) ICSB=-1
      NIPI=1
      IF(NCHAN.EQ.1) NIPI=4
      IF(NCHAN.NE.1) LC=1
      IF(NCHAN.EQ.1) LC=3
C
C     INITIALIZATION POTENTIALS
C
      CALL INIT
C
      DO 25 II=1,10
*     DO 25 II=1,4
C
      DO 24 IP=1,4
      VCAA(IP)  =ZERO
      VSIGAA(IP)=ZERO
      VTENAA(IP)=ZERO
      VSOAA(IP) =ZERO
      VSO2AA(IP)=ZERO
      VASOAA(IP)=ZERO
      FIAA(IP)  =ZERO
      DFIAA(IP) =ZERO
**    DDFIAA(IP)=ZERO
      FISAA(IP) =ZERO
      DFISAA(IP)=ZERO
**    DDFISA(IP)=ZERO
      FITAA(IP) =ZERO
      DFITAA(IP)=ZERO
**    DDFITA(IP)=ZERO
      VCESC(IP) = ZERO
      VSESC(IP) = ZERO
      VTESC(IP) = ZERO
      VOESC(IP) = ZERO
      VAESC(IP) = ZERO
  24  VNNAA(IP) = ZERO
      esccnn = zero
      escsnn = zero
      esctnn = zero
      esconn = zero

C
      GOTO(26,27,28,29,291,292,15,16,17,18),II
*     GOTO(26,27,28,29),II
  26  IF(IPSC.EQ.1) CALL PSSCAL(X,ICSB,NLOC,LC)       
**26  IF(IPSC.EQ.1) CALL PSSCAL02(X,ICSB,NLOC,LC)       
* 26  IF(IPSC.EQ.1) CALL PSSCAL3(X,NADIA,ICSB,NLOC,NCHAN)
      GOTO 30
  27  IF(IVCT.EQ.1) CALL VECTOR(X,ICSB,NLOC,LC)        
* 27  IF(IVCT.EQ.1) CALL VECTOR3(X,NADIA,ICSB,NLOC,NCHAN)
      GOTO 30
  28  IF(ISCL.EQ.1) THEN
       CALL SCALAR(X,ICSB,NLOC,LC)        
       CALL SCALAR2(X,ICSB,NLOC,LC)    
      ENDIF
* 28  IF(ISCL.EQ.1) CALL SCALAR3(X,NADIA,ICSB,NLOC,NCHAN)
      GOTO 30
  29  IF(IDIF.EQ.1) CALL DIFRAC(X,ICSB,NLOC,LC)       
      GOTO 30
  291 IF(IAXI.EQ.1) CALL AXIAL(X,ICSB,NLOC,LC,P)       
      GOTO 30
  292 IF(ITEN.EQ.1) CALL TENSOR(X,ICSB,NLOC,LC,P)       
      GOTO 30
  15  IF(ITPS.EQ.1) CALL YNPSPS(X,INA,IPV,IOFF,APV)                         
      GOTO 40
  16  IF(IPAIR.EQ.1) CALL YNPAIR(X,INA,IPV,APV)                              
      GOTO 40
  17  IF(IPIBE.EQ.1) CALL YNPIBE(X,INA,IPV,IOFF)                        
      GOTO 40
  18  IF(IPSBE.EQ.1) CALL YNPSBE(X,INA,IPV,IOFF)                        
      GOTO 40

  30  IF(MOD(ICALL,5).EQ.1) THEN
      IF(NS.EQ.3) THEN
      IF(II.LE.4) THEN
*     IF(ICALL.EQ.1) WRITE(N2,311) TYPM(II),X,(TYPV(KK),KK=1,7)
      WRITE(N2,311) TYPM(II),X,(TYPV(KK),KK=1,7)
 311  FORMAT(/,A4,' X=',F7.4,' PIM-1',' POTENTIALS:',/,
     .         /,6X,7(3X,A4,3X),/)
      DO 31 IP=NIPI,4
  31  WRITE(N2,32) CHAN(IP),VCAA(IP), VSIGAA(IP),VTENAA(IP),
     .             VSOAA(IP),VASOAA(IP),FIAA(IP),DFIAA(IP)
*    .             VSOAA(IP),VSO2AA(IP),FIAA(IP),DFIAA(IP),DDFIAA(IP)
  32  FORMAT(1X,A4,2X,7(D9.3,1X),/)
      ENDIF
      ENDIF
      ENDIF

  40  IF(MOD(ICALL,5).EQ.1) THEN
      IF(NS.EQ.3) THEN
      IF(II.GE.5) THEN
*     IF(ICALL.EQ.1) WRITE(N2,411) TYPM(II),X,(TYPV(KK),KK=1,4)
      WRITE(N2,411) TYPM(II),X,(TYPV(KK),KK=1,4)
 411  FORMAT(/,A4,' X=',F7.4,' PIM-1',' POTENTIALS:',/,
     .         /,6X,4(3X,A4,3X),/)
      DO 41 IP=NIPI,4
  41  IF(MOD(ICALL,5).EQ.1)
     .WRITE(N2,42) CHAN(IP),VCESC(IP), VSESC(IP),VTESC(IP),
     .             VOESC(IP)
  42  FORMAT(1X,A4,2X,4(D9.3,1X),/)
      ENDIF
      ENDIF
      ENDIF
C
      VCNN    = VCNN   + VNNAA(1)
      VSIGNN  = VSIGNN + VNNAA(2)
      VTENNN  = VTENNN + VNNAA(3)
      VSONN   = VSONN  + VNNAA(4)
      IF(IPAIR.EQ.1) THEN
      VCNN    = VCNN   + ESCCNN   
      VSIGNN  = VSIGNN + ESCSNN   
      VTENNN  = VTENNN + ESCTNN  
      VSONN   = VSONN  + ESCONN  
      ENDIF
      IF(NCHAN.EQ.1) GOTO 33
      VCLL  =VCLL   + VCAA(1) + VCESC(1)
      VSIGLL=VSIGLL + VSIGAA(1) + VSESC(1)
      VTENLL=VTENLL + VTENAA(1) + VTESC(1)
      VSOLL =VSOLL  + VSOAA(1)  + VOESC(1)
      VSO2LL=VSO2LL + VSO2AA(1)
      VASOLL=VASOLL + VASOAA(1) + VAESC(1)
      VCLS  =VCLS   + VCAA(3)   + VCESC(3)
      VSIGLS=VSIGLS + VSIGAA(3) + VSESC(3)
      VTENLS=VTENLS + VTENAA(3) + VTESC(3)
      VSOLS =VSOLS  + VSOAA(3)  + VOESC(3)
      VSO2LS=VSO2LS + VSO2AA(3)
      VASOLS=VASOLS + VASOAA(3)
      VCSS  =VCSS   + VCAA(2)   + VCESC(2)
      VSIGSS=VSIGSS + VSIGAA(2) + VSESC(2)
      VTENSS=VTENSS + VTENAA(2) + VTESC(2)
      VSOSS =VSOSS  + VSOAA(2)  + VOESC(2)
      VSO2SS=VSO2SS + VSO2AA(2)
      VASOSS=VASOSS + VASOAA(2) + VAESC(2)
      FILL  =FILL   + FIAA(1)
      FILS  =FILS   + FIAA(3)
      FISS  =FISS   + FIAA(2)
      DFILL =DFILL  + DFIAA(1)
      DFILS =DFILS  + DFIAA(3)
      DFISS =DFISS  + DFIAA(2)
**    DDFILL=DDFILL + DDFIAA(1)
**    DDFILS=DDFILS + DDFIAA(3)
**    DDFISS=DDFISS + DDFIAA(2)
      FISLL  =FISLL   + FISAA(1)
      FISLS  =FISLS   + FISAA(3)
      FISSS  =FISSS   + FISAA(2)
      DFISLL =DFISLL  + DFISAA(1)
      DFISLS =DFISLS  + DFISAA(3)
      DFISSS =DFISSS  + DFISAA(2)
**    DDFSLL=DDFSLL + DDFISA(1)
**    DDFSLS=DDFSLS + DDFISA(3)
**    DDFSSS=DDFSSS + DDFISA(2)
  33  VCDR  =VCDR   + VCAA(4) + VCESC(4)
      VSIGDR=VSIGDR + VSIGAA(4) + VSESC(4)
      VTENDR=VTENDR + VTENAA(4) + VTESC(4)
      VSODR =VSODR  + VSOAA(4)  + VOESC(4)
      VSO2DR=VSO2DR + VSO2AA(4)
      VASODR=VASODR + VASOAA(4) + VAESC(4)
      FIDR  =FIDR   + FIAA(4)
      DFIDR =DFIDR  + DFIAA(4)
**    DDFIDR=DDFIDR + DDFIAA(4)
      FISDR  =FISDR   + FISAA(4)
20    DFISDR =DFISDR  + DFISAA(4)
**20  DDFSDR=DDFSDR + DDFISA(4)
  25  CONTINUE
      IF((JSPIN.EQ.0.AND.MOD(ICALLS,5).EQ.1).or.                           
     .(JSPIN.EQ.1.AND.MOD(ICALLT,5).EQ.1)) THEN                        
      IF(JSPIN.EQ.0) WRITE(*,99)
  99  FORMAT(72('-'))
**    IF(MOD(ICALL,2).EQ.1) THEN
      IF(NS.GE.2) THEN
      XFM = X*197.327053D0/PIM
      WRITE(N2,312) XFM,JSPIN,LINT,(TYPV(KK),KK=1,7)
**    WRITE(N2,312) XFM,JSPIN,LINT,(TYPV2(KK),KK=1,7)
 312  FORMAT(/,' XFM=',F7.4,' FM',' JSPIN=',I2,' LINT=',I2,
     .' TOTAL POTENTIALS:',/, /,6X,7(3X,A4,3X),/)
      IF(NCHAN.NE.1) THEN
*     WRITE(N2,51)  VCLL, VSIGLL,VTENLL,VSOLL,VSO2LL,FILL,DFILL,DDFILL
*     WRITE(N2,52)  VCSS, VSIGSS,VTENSS,VSOSS,VSO2SS,FISS,DFISS,DDFISS
*     WRITE(N2,53)  VCLS, VSIGLS,VTENLS,VSOLS,VSO2LS,FILS,DFILS,DDFILS
*     WRITE(N2,54)  VCDR, VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR

      WRITE(N2,51)  VCLL, VSIGLL,VTENLL,VSOLL,VASOLL,FILL,FISLL
      WRITE(N2,52)  VCSS, VSIGSS,VTENSS,VSOSS,VASOSS,FISS,FISSS
      WRITE(N2,53)  VCLS, VSIGLS,VTENLS,VSOLS,VASOLS,FILS,FISLS
      WRITE(N2,54)  VCDR, VSIGDR,VTENDR,VSODR,VASODR,FIDR,FISDR

cc    WRITE(N2,51)  VCLL, VSIGLL,VTENLL,FILL,FISLL,DFILL,DFISLL
cc    WRITE(N2,52)  VCSS, VSIGSS,VTENSS,FISS,FISSS,DFISS,DFISSS
cc    WRITE(N2,53)  VCLS, VSIGLS,VTENLS,FILS,FISLS,DFILS,DFISLS
cc    WRITE(N2,54)  VCDR, VSIGDR,VTENDR,FIDR,FISDR,DFIDR,DFISDR
      ELSE
*     WRITE(N2,55)  VCDR, VSIGDR,VTENDR,VSODR,VSO2DR,FIDR,DFIDR,DDFIDR
      WRITE(N2,55)  VCDR, VSIGDR,VTENDR,VSODR,VASODR,FIDR,FISDR
cc    WRITE(N2,55)  VCDR, VSIGDR,VTENDR,FIDR,FISDR,DFIDR,DFISDR
      ENDIF
      ENDIF
      ENDIF
  51  FORMAT(1X,' LL :',7(D9.3,1X),/)
  52  FORMAT(1X,' SS :',7(D9.3,1X),/)
  53  FORMAT(1X,' LS :',7(D9.3,1X),/)
  54  FORMAT(1X,' DR :',7(D9.3,1X),/)
  55  FORMAT(1X,' DR :',7(D9.3,1X),/)
C
      RETURN
      END
C**********************************************************************
C
      SUBROUTINE INIT
C
C**********************************************************************
      IMPLICIT REAL *8(A-H,O-Z)
      COMMON/POT/V(48)
**    COMMON/POT/V(53)
      DATA ZERO/0.D0/
C
      DO 24 IP=1,48
**    DO 24 IP=1,53
  24  V(IP) = ZERO
      RETURN
      END
C *************************************************************
      SUBROUTINE CNSTR97(ELPHID,GEPS,GSST,GEPSNW,GSSTNW,NCASE)
C *************************************************************
C     ADAPTED FOR THE VERSION WITH 2 SCALAR NONETS !! (SEPT. 2000)
C -------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/ALLSC/ASI,BSI,AM1SI,AM2SI,ASST,BSST,AMSST,AM2ST,AMD,AMSCK
      DATA SR2/1.4142136D0/,SR3/1.732051D0/,PIM/138.041D0/                
      DATA NS/0/,AMEPS2/1370.D0/,AMD2/1450.D0/,AMSST2/1580.D0/
CC
C     COMPUTATION ALTERNATIVE GEPS,GSST FROM THE ELLIPTIC CONSTRAINT:
C     VOLUME INTEGRAL VNN CONSTANT:
C     PRINT 200
C 200 FORMAT(/,' ELLIPTIC CONSTRAINT= PRESERVING VOL.INT NN:',/)
      PI   =DACOS(-1.D0)
      ELPHI=PI*ELPHID/180.D0
      COSFI=DCOS(ELPHI)
      SINFI=DSIN(ELPHI)
      IF(NCASE.EQ.1) THEN
      BMSI =ASI*(PIM/AM1SI)**2 + BSI*(PIM/AM2SI)**2
      BMSST=(PIM/AMSST)**2
      CF   = DSQRT(BMSI*GEPS**2 + BMSST*GSST**2)
      ENDIF
      IF(NCASE.EQ.2) THEN
      BMSI =(PIM/AMD)**2 
      BMSST=(PIM/AMD2)**2
      CF   = DSQRT(BMSI*GEPS**2 + BMSST*GSST**2)
      ENDIF
      IF(NCASE.EQ.3) THEN
      BMSI =(PIM/AMEPS2)**2 
      BMSST=(PIM/AMSST2)**2
      CF   = DSQRT(BMSI*GEPS**2 + BMSST*GSST**2)
      ENDIF
      IF(NCASE.EQ.5) THEN
      BMSI =ASI*(PIM/AM1SI)**2 + BSI*(PIM/AM2SI)**2
      BMSST=(PIM/AMEPS2)**2
      CF   = DSQRT(BMSI*GEPS**2 + BMSST*GSST**2)
      ENDIF

C     STANDARD ELLIPSE PARAMETERS, X^2/A^2 + Y^2/B^2 = :
      AA = 1.D0/DSQRT(BMSI)
      BB = 1.D0/DSQRT(BMSST)
      XX = GEPS/CF
      YY = GSST/CF
      
C     POLAR COORDINATES, (ELPHI,RR):
      EPSI2 = 1.D0-(BB/AA)**2
      RR    = BB/DSQRT(1.D0-EPSI2*COSFI**2)

      GEPSNW= CF*RR*COSFI
      GSSTNW= CF*RR*SINFI

      IF(NS.EQ.1) THEN
C     COMPUTATION ELPHI0: CORRESPONDS TO INPUT GEPS,GSST:
      ARG = GSST/GEPS
      IF(ARG.GE.0) ELPHI0=DATAN(ARG)*180.D0/PI
C     IF(ARG.LT.0) ELPHI0=360.D0+DATAN(ARG)*180.D0/PI
      IF(ARG.LT.0) ELPHI0=DATAN(ARG)*180.D0/PI

      IF(GSST.LT.0.D0.AND.GEPS.LT.0.D0) 
     .ELPHI0=ELPHI0 - 180.D0
      
      IF(NCASE.EQ.1)        
     .PRINT 111, ELPHID,GEPSNW,GSSTNW,ELPHI0
  111 FORMAT(' IN CNSTR97: ELPHID=',F8.3,' GEPSNW=',D12.5,
     .' GSSTNW=',D12.5,/,12X,' ELPHI0=',D12.5,/)
      IF(NCASE.EQ.2)        
     .PRINT 112, ELPHID,GEPSNW,GSSTNW,ELPHI0
  112 FORMAT(' IN CNSTR97: PSIS8D=',F8.3,' GDELNW=',D12.5,
     .' GDEL2NW=',D12.5,/,12X,' PSIS80=',D12.5,/)
      IF(NCASE.EQ.3)        
     .PRINT 113, ELPHID,GEPSNW,GSSTNW,ELPHI0
  113 FORMAT(' IN CNSTR97: PSIS1D=',F8.3,' GEPS2NW=',D12.5,
     .' GSST2NW=',D12.5,/,12X,' PSIS10=',D12.5,/)
      IF(NCASE.EQ.5)        
     .PRINT 115, ELPHID,GEPSNW,GSSTNW,ELPHI0
  115 FORMAT(' IN CNSTR97: PSIS1D=',F8.3,' GEPSNW=',D12.5,
     .' GEPS2NW=',D12.5,/,12X,' PSIS10=',D12.5,/)

      ENDIF
      RETURN
      END
C **************************************************************
      SUBROUTINE TRANLP(TRANS,TRAND)
C **************************************************************
C     LAMBDA-PROTON SYSTEM VERSION
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION TRANS(3,3),TRAND(3,3)
      DATA SR2/1.4142136D0/,SR3/1.732051D0/
      SR2 = DSQRT(2.D0)
      SR3 = DSQRT(3.D0)
      TRANS(1,1)=1.0D0
      TRANS(1,2)=SR2/SR3
      TRANS(2,1)=SR2/SR3
      TRANS(1,3)=-1.D0/SR3
      TRANS(3,1)=-1.D0/SR3
      TRANS(2,2)=2.D0/3
      TRANS(2,3)=-SR2/3
      TRANS(3,2)=-SR2/3
      TRANS(3,3)=1.D0/3
C
      TRAND(1,1)=0.D0
      TRAND(1,2)=0.D0
      TRAND(2,1)=0.D0
      TRAND(1,3)=0.D0
      TRAND(3,1)=0.D0
      TRAND(2,2)=1.D0/3
      TRAND(2,3)=SR2/3
      TRAND(3,2)=SR2/3
      TRAND(3,3)=2.D0/3
      RETURN
      END
C **************************************************************
      SUBROUTINE TRANLN(TRANS,TRAND)
C **************************************************************
C     LAMBDA-NEUTRON SYSTEM VERSION
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION TRANS(3,3),TRAND(3,3)
      DATA SR2/1.4142136D0/,SR3/1.732051D0/
      SR2 = DSQRT(2.D0)
      SR3 = DSQRT(3.D0)
      TRANS(1,1)=1.0D0
      TRANS(1,2)=1.D0/SR3
      TRANS(2,1)=1.D0/SR3
      TRANS(1,3)=-SR2/SR3
      TRANS(3,1)=-SR2/SR3
      TRANS(2,2)=1.D0/3
      TRANS(2,3)=-SR2/3
      TRANS(3,2)=-SR2/3
      TRANS(3,3)=2.D0/3
C
      TRAND(1,1)=0.D0
      TRAND(1,2)=0.D0
      TRAND(2,1)=0.D0
      TRAND(1,3)=0.D0
      TRAND(3,1)=0.D0
      TRAND(2,2)=2.D0/3
      TRAND(2,3)=SR2/3
      TRAND(3,2)=SR2/3
      TRAND(3,3)=1.D0/3
      RETURN
      END

C **********************************************************************
      SUBROUTINE GERSTPAR           
C **********************************************************************
C     COMPUTES TWO-POLE APPRXIMATION PARAMETERS FOR EPSILON-MESON,
C     USING THE Bryan-Gersten FORMULAS, PHYS.REV. D6 (1972) 341
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/ALLSC/ASI,BSI,AM1SI,AM2SI,ASST,BSST,AMSST,AM2ST,AMD,AMSCK
      COMMON/MASSES/PIM,AM(3),AMY(3),REDM(3),PLAB,AK(3),AKS(3),TEM(3)
      COMMON/PRMTRS/PAR(20,8)

      AME  = PAR(10,5)
      GAME = PAR(11,5)
      PIM2 = PIM*PIM
      AME2 = AME*AME
      AM32 = AME2*(DSQRT(AME2+4*PIM2)-2*PIM)
      AM32 = 4*PIM*AM32/(AME2+8*PIM2-4*PIM*DSQRT(AME2+4*PIM2))
      GAM  = GAME*AME/DSQRT(AME2-4*PIM2)

      X = (AME2+2*PIM*GAM)*AM32
      Y = AME2+AM32*(1.D0+0.25D0*GAM/PIM)+2*PIM*GAM
      Y = 0.5D0*Y
      AM12 = Y-DSQRT(Y*Y-X)
      AM22 = Y+DSQRT(Y*Y-X)

      ASI  = (AM12-AM32)/(AM12-AM22)
      BSI  = 1.D0-ASI
      AM1SI= DSQRT(AM12)
      AM2SI= DSQRT(AM22)

      RETURN
      END

C **********************************************************************
      SUBROUTINE FRMINIT            
C **********************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/FORMF/ALAM,ALMP1,ALMV1,ALMS1,ALMP0,ALMV0,ALMS0,             
     .             ALMKA,ALMKS,ALMKP
      COMMON/PRMTRS/ PAR(20,8)

C     COMMON FORM FACTOR FOR PS, VECT, AND SCALA RESPECTIVELY:
      ALMP0 = ALMP1
      ALMV0 = ALMV1
      ALMS0 = ALMS1

C     PUTTING THINGS IN PARAMETER MATRIX:
      PAR(9,2)=ALMP0
      PAR(9,3)=ALMV0
      PAR(9,5)=ALMS0
      RETURN
      END
      










